"""
Python module generated from Java source file org.bukkit.event.world.AsyncStructureSpawnEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import World
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.world import *
from org.bukkit.generator.structure import Structure
from org.bukkit.util import BoundingBox
from typing import Any, Callable, Iterable, Tuple


class AsyncStructureSpawnEvent(WorldEvent, Cancellable):
    """
    Called when a Structure is naturally generated in the world.
    """

    def __init__(self, world: "World", structure: "Structure", boundingBox: "BoundingBox", chunkX: int, chunkZ: int):
        ...


    def getStructure(self) -> "Structure":
        """
        Get the structure reference that is generated.

        Returns
        - the structure
        """
        ...


    def getBoundingBox(self) -> "BoundingBox":
        """
        Get the bounding box of the structure.

        Returns
        - the bounding box
        """
        ...


    def getChunkX(self) -> int:
        """
        Get the x coordinate of the origin chunk of the structure.
        
        **Note, it is not safe to attempt to retrieve or interact with this
        chunk. This event is informative only!**

        Returns
        - the chunk x coordinate
        """
        ...


    def getChunkZ(self) -> int:
        """
        Get the z coordinate of the origin chunk of the structure.
        
        **Note, it is not safe to attempt to retrieve or interact with this
        chunk. This event is informative only!**

        Returns
        - the chunk z coordinate
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.AbstractListeningExecutorService

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.util.concurrent import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import CheckReturnValue
from java.util.concurrent import AbstractExecutorService
from java.util.concurrent import Callable
from java.util.concurrent import RunnableFuture
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class AbstractListeningExecutorService(AbstractExecutorService, ListeningExecutorService):
    """
    Abstract ListeningExecutorService implementation that creates ListenableFuture
    instances for each Runnable and Callable submitted to it. These tasks are run
    with the abstract .execute execute(Runnable) method.
    
    In addition to .execute, subclasses must implement all methods related to shutdown and
    termination.

    Author(s)
    - Chris Povirk

    Since
    - 14.0
    """

    def submit(self, task: "Runnable") -> "ListenableFuture"[Any]:
        ...


    def submit(self, task: "Runnable", result: "T") -> "ListenableFuture"["T"]:
        ...


    def submit(self, task: "Callable"["T"]) -> "ListenableFuture"["T"]:
        ...



"""
Python module generated from Java source file com.google.common.cache.RemovalCause

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.cache import *
from enum import Enum
from java.util import Iterator
from java.util.concurrent import ConcurrentMap
from typing import Any, Callable, Iterable, Tuple


class RemovalCause(Enum):
    """
    The reason why a cached entry was removed.

    Author(s)
    - Charles Fry

    Since
    - 10.0
    """

    EXPLICIT = 0
    """
    The entry was manually removed by the user. This can result from the user invoking Cache.invalidate, Cache.invalidateAll(Iterable), Cache.invalidateAll(), Map.remove, ConcurrentMap.remove, or Iterator.remove.
    """
    REPLACED = 1
    """
    The entry itself was not actually removed, but its value was replaced by the user. This can
    result from the user invoking Cache.put, LoadingCache.refresh, Map.put,
    Map.putAll, ConcurrentMap.replace(Object, Object), or ConcurrentMap.replace(Object, Object, Object).
    """
    COLLECTED = 2
    """
    The entry was removed automatically because its key or value was garbage-collected. This can
    occur when using CacheBuilder.weakKeys, CacheBuilder.weakValues, or CacheBuilder.softValues.
    """
    EXPIRED = 3
    """
    The entry's expiration timestamp has passed. This can occur when using CacheBuilder.expireAfterWrite or CacheBuilder.expireAfterAccess.
    """
    SIZE = 4
    """
    The entry was evicted due to size constraints. This can occur when using CacheBuilder.maximumSize or CacheBuilder.maximumWeight.
    """



"""
Python module generated from Java source file org.joml.QuaterniondInterpolator

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.joml import *
from typing import Any, Callable, Iterable, Tuple


class QuaterniondInterpolator:
    """
    Computes the weighted average of multiple rotations represented as Quaterniond instances.
    
    Instances of this class are *not* thread-safe.

    Author(s)
    - Kai Burjack
    """

    def computeWeightedAverage(self, qs: list["Quaterniond"], weights: list[float], maxSvdIterations: int, dest: "Quaterniond") -> "Quaterniond":
        """
        Compute the weighted average of all of the quaternions given in `qs` using the specified interpolation factors `weights`, and store the result in `dest`.

        Arguments
        - qs: the quaternions to interpolate over
        - weights: the weights of each individual quaternion in `qs`
        - maxSvdIterations: the maximum number of iterations in the Singular Value Decomposition step used by this method
        - dest: will hold the result

        Returns
        - dest
        """
        ...



"""
Python module generated from Java source file java.nio.file.attribute.PosixFilePermissions

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.nio.file.attribute import *
from java.util import *
from typing import Any, Callable, Iterable, Tuple


class PosixFilePermissions:

    @staticmethod
    def toString(perms: set["PosixFilePermission"]) -> str:
        """
        Returns the `String` representation of a set of permissions. It
        is guaranteed that the returned `String` can be parsed by the
        .fromString method.
        
         If the set contains `null` or elements that are not of type
        `PosixFilePermission` then these elements are ignored.

        Arguments
        - perms: the set of permissions

        Returns
        - the string representation of the permission set
        """
        ...


    @staticmethod
    def fromString(perms: str) -> set["PosixFilePermission"]:
        """
        Returns the set of permissions corresponding to a given `String`
        representation.
        
         The `perms` parameter is a `String` representing the
        permissions. It has 9 characters that are interpreted as three sets of
        three. The first set refers to the owner's permissions; the next to the
        group permissions and the last to others. Within each set, the first
        character is `'r'` to indicate permission to read, the second
        character is `'w'` to indicate permission to write, and the third
        character is `'x'` for execute permission. Where a permission is
        not set then the corresponding character is set to `'-'`.
        
         **Usage Example:**
        Suppose we require the set of permissions that indicate the owner has read,
        write, and execute permissions, the group has read and execute permissions
        and others have none.
        ```
          Set&lt;PosixFilePermission&gt; perms = PosixFilePermissions.fromString("rwxr-x---");
        ```

        Arguments
        - perms: string representing a set of permissions

        Returns
        - the resulting set of permissions

        Raises
        - IllegalArgumentException: if the string cannot be converted to a set of permissions

        See
        - .toString(Set)
        """
        ...


    @staticmethod
    def asFileAttribute(perms: set["PosixFilePermission"]) -> "FileAttribute"[set["PosixFilePermission"]]:
        """
        Creates a FileAttribute, encapsulating a copy of the given file
        permissions, suitable for passing to the java.nio.file.Files.createFile
        createFile or java.nio.file.Files.createDirectory createDirectory
        methods.

        Arguments
        - perms: the set of permissions

        Returns
        - an attribute encapsulating the given file permissions with
                 FileAttribute.name name `"posix:permissions"`

        Raises
        - ClassCastException: if the set contains elements that are not of type `PosixFilePermission`
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.entity.PigZapEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Collections
from org.bukkit.entity import Entity
from org.bukkit.entity import LightningStrike
from org.bukkit.entity import Pig
from org.bukkit.entity import PigZombie
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.entity import *
from typing import Any, Callable, Iterable, Tuple


class PigZapEvent(EntityTransformEvent, Cancellable):
    """
    Stores data for pigs being zapped
    """

    def __init__(self, pig: "Pig", bolt: "LightningStrike", pigzombie: "PigZombie"):
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getEntity(self) -> "Pig":
        ...


    def getLightning(self) -> "LightningStrike":
        """
        Gets the bolt which is striking the pig.

        Returns
        - lightning entity
        """
        ...


    def getPigZombie(self) -> "PigZombie":
        """
        Gets the zombie pig that will replace the pig, provided the event is
        not cancelled first.

        Returns
        - resulting entity

        Deprecated
        - use EntityTransformEvent.getTransformedEntity()
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.plugin.PluginLogger

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.plugin import *
from typing import Any, Callable, Iterable, Tuple


class PluginLogger(Logger):
    """
    The PluginLogger class is a modified Logger that prepends all
    logging calls with the name of the plugin doing the logging. The API for
    PluginLogger is exactly the same as Logger.

    See
    - Logger
    """

    def __init__(self, context: "Plugin"):
        """
        Creates a new PluginLogger that extracts the name from a plugin.

        Arguments
        - context: A reference to the plugin
        """
        ...


    def log(self, logRecord: "LogRecord") -> None:
        ...



"""
Python module generated from Java source file com.google.common.hash.HashCode

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from com.google.common.hash import *
from com.google.common.primitives import Ints
from com.google.common.primitives import UnsignedInts
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.io import Serializable
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class HashCode:
    """
    An immutable hash code of arbitrary bit length.

    Author(s)
    - Kurt Alfred Kluever

    Since
    - 11.0
    """

    def bits(self) -> int:
        """
        Returns the number of bits in this hash code; a positive multiple of 8.
        """
        ...


    def asInt(self) -> int:
        """
        Returns the first four bytes of .asBytes() this hashcode's bytes, converted to an
        `int` value in little-endian order.

        Raises
        - IllegalStateException: if `bits() < 32`
        """
        ...


    def asLong(self) -> int:
        """
        Returns the first eight bytes of .asBytes() this hashcode's bytes, converted to a
        `long` value in little-endian order.

        Raises
        - IllegalStateException: if `bits() < 64`
        """
        ...


    def padToLong(self) -> int:
        """
        If this hashcode has enough bits, returns `asLong()`, otherwise returns a `long`
        value with `asBytes()` as the least-significant bytes and `0x00` as the remaining
        most-significant bytes.

        Since
        - 14.0 (since 11.0 as `Hashing.padToLong(HashCode)`)
        """
        ...


    def asBytes(self) -> list[int]:
        ...


    def writeBytesTo(self, dest: list[int], offset: int, maxLength: int) -> int:
        """
        Copies bytes from this hash code into `dest`.

        Arguments
        - dest: the byte array into which the hash code will be written
        - offset: the start offset in the data
        - maxLength: the maximum number of bytes to write

        Returns
        - the number of bytes written to `dest`

        Raises
        - IndexOutOfBoundsException: if there is not enough room in `dest`
        """
        ...


    @staticmethod
    def fromInt(hash: int) -> "HashCode":
        """
        Creates a 32-bit `HashCode` representation of the given int value. The underlying bytes
        are interpreted in little endian order.

        Since
        - 15.0 (since 12.0 in HashCodes)
        """
        ...


    @staticmethod
    def fromLong(hash: int) -> "HashCode":
        """
        Creates a 64-bit `HashCode` representation of the given long value. The underlying bytes
        are interpreted in little endian order.

        Since
        - 15.0 (since 12.0 in HashCodes)
        """
        ...


    @staticmethod
    def fromBytes(bytes: list[int]) -> "HashCode":
        """
        Creates a `HashCode` from a byte array. The array is defensively copied to preserve the
        immutability contract of `HashCode`. The array cannot be empty.

        Since
        - 15.0 (since 12.0 in HashCodes)
        """
        ...


    @staticmethod
    def fromString(string: str) -> "HashCode":
        """
        Creates a `HashCode` from a hexadecimal (`base 16`) encoded string. The string must
        be at least 2 characters long, and contain only valid, lower-cased hexadecimal characters.
        
        This method accepts the exact format generated by .toString. If you require more
        lenient `base 16` decoding, please use com.google.common.io.BaseEncoding.decode
        (and pass the result to .fromBytes).

        Since
        - 15.0
        """
        ...


    def equals(self, object: "Object") -> bool:
        """
        Returns `True` if `object` is a HashCode instance with the identical byte
        representation to this hash code.
        
        **Security note:** this method uses a constant-time (not short-circuiting) implementation
        to protect against <a href="http://en.wikipedia.org/wiki/Timing_attack">timing attacks</a>.
        """
        ...


    def hashCode(self) -> int:
        """
        Returns a "Java hash code" for this `HashCode` instance; this is well-defined (so, for
        example, you can safely put `HashCode` instances into a `HashSet`) but is otherwise
        probably not what you want to use.
        """
        ...


    def toString(self) -> str:
        """
        Returns a string containing each byte of .asBytes, in order, as a two-digit unsigned
        hexadecimal number in lower case.
        
        Note that if the output is considered to be a single hexadecimal number, whether this string
        is big-endian or little-endian depends on the byte order of .asBytes. This may be
        surprising for implementations of `HashCode` that represent the number in big-endian
        since everything else in the hashing API uniformly treats multibyte values as little-endian.
        
        To create a `HashCode` from its string representation, see .fromString.
        """
        ...



"""
Python module generated from Java source file java.io.UnsupportedEncodingException

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import *
from typing import Any, Callable, Iterable, Tuple


class UnsupportedEncodingException(IOException):
    """
    The Character Encoding is not supported.

    Author(s)
    - Asmus Freytag

    Since
    - 1.1
    """

    def __init__(self):
        """
        Constructs an UnsupportedEncodingException without a detail message.
        """
        ...


    def __init__(self, s: str):
        """
        Constructs an UnsupportedEncodingException with a detail message.

        Arguments
        - s: Describes the reason for the exception.
        """
        ...



"""
Python module generated from Java source file com.google.common.math.DoubleMath

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.math import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.math import BigInteger
from java.math import RoundingMode
from java.util import Iterator
from typing import Any, Callable, Iterable, Tuple


class DoubleMath:
    """
    A class for arithmetic on doubles that is not covered by java.lang.Math.

    Author(s)
    - Louis Wasserman

    Since
    - 11.0
    """

    @staticmethod
    def roundToInt(x: float, mode: "RoundingMode") -> int:
        """
        Returns the `int` value that is equal to `x` rounded with the specified rounding
        mode, if possible.

        Raises
        - ArithmeticException: if
            
              - `x` is infinite or NaN
              - `x`, after being rounded to a mathematical integer using the specified rounding
                  mode, is either less than `Integer.MIN_VALUE` or greater than `Integer.MAX_VALUE`
              - `x` is not a mathematical integer and `mode` is RoundingMode.UNNECESSARY
            
        """
        ...


    @staticmethod
    def roundToLong(x: float, mode: "RoundingMode") -> int:
        """
        Returns the `long` value that is equal to `x` rounded with the specified rounding
        mode, if possible.

        Raises
        - ArithmeticException: if
            
              - `x` is infinite or NaN
              - `x`, after being rounded to a mathematical integer using the specified rounding
                  mode, is either less than `Long.MIN_VALUE` or greater than `Long.MAX_VALUE`
              - `x` is not a mathematical integer and `mode` is RoundingMode.UNNECESSARY
            
        """
        ...


    @staticmethod
    def roundToBigInteger(x: float, mode: "RoundingMode") -> "BigInteger":
        ...


    @staticmethod
    def isPowerOfTwo(x: float) -> bool:
        """
        Returns `True` if `x` is exactly equal to `2^k` for some finite integer
        `k`.
        """
        ...


    @staticmethod
    def log2(x: float) -> float:
        """
        Returns the base 2 logarithm of a double value.
        
        Special cases:
        
        
          - If `x` is NaN or less than zero, the result is NaN.
          - If `x` is positive infinity, the result is positive infinity.
          - If `x` is positive or negative zero, the result is negative infinity.
        
        
        The computed result is within 1 ulp of the exact result.
        
        If the result of this method will be immediately rounded to an `int`, .log2(double, RoundingMode) is faster.
        """
        ...


    @staticmethod
    def log2(x: float, mode: "RoundingMode") -> int:
        """
        Returns the base 2 logarithm of a double value, rounded with the specified rounding mode to an
        `int`.
        
        Regardless of the rounding mode, this is faster than `(int) log2(x)`.

        Raises
        - IllegalArgumentException: if `x <= 0.0`, `x` is NaN, or `x` is
            infinite
        """
        ...


    @staticmethod
    def isMathematicalInteger(x: float) -> bool:
        """
        Returns `True` if `x` represents a mathematical integer.
        
        This is equivalent to, but not necessarily implemented as, the expression `!Double.isNaN(x) && !Double.isInfinite(x) && x == Math.rint(x)`.
        """
        ...


    @staticmethod
    def factorial(n: int) -> float:
        """
        Returns `n!`, that is, the product of the first `n` positive integers, `1` if
        `n == 0`, or `n!`, or Double.POSITIVE_INFINITY if `n! >
        Double.MAX_VALUE`.
        
        The result is within 1 ulp of the True value.

        Raises
        - IllegalArgumentException: if `n < 0`
        """
        ...


    @staticmethod
    def fuzzyEquals(a: float, b: float, tolerance: float) -> bool:
        """
        Returns `True` if `a` and `b` are within `tolerance` of each other.
        
        Technically speaking, this is equivalent to `Math.abs(a - b) <= tolerance ||
        Double.valueOf(a).equals(Double.valueOf(b))`.
        
        Notable special cases include:
        
        
          - All NaNs are fuzzily equal.
          - If `a == b`, then `a` and `b` are always fuzzily equal.
          - Positive and negative zero are always fuzzily equal.
          - If `tolerance` is zero, and neither `a` nor `b` is NaN, then `a`
              and `b` are fuzzily equal if and only if `a == b`.
          - With Double.POSITIVE_INFINITY tolerance, all non-NaN values are fuzzily equal.
          - With finite tolerance, `Double.POSITIVE_INFINITY` and `Double.NEGATIVE_INFINITY` are fuzzily equal only to themselves.
        
        
        This is reflexive and symmetric, but *not* transitive, so it is *not* an
        equivalence relation and *not* suitable for use in Object.equals
        implementations.

        Raises
        - IllegalArgumentException: if `tolerance` is `< 0` or NaN

        Since
        - 13.0
        """
        ...


    @staticmethod
    def fuzzyCompare(a: float, b: float, tolerance: float) -> int:
        """
        Compares `a` and `b` "fuzzily," with a tolerance for nearly-equal values.
        
        This method is equivalent to `fuzzyEquals(a, b, tolerance) ? 0 : Double.compare(a,
        b)`. In particular, like Double.compare(double, double), it treats all NaN values as
        equal and greater than all other values (including Double.POSITIVE_INFINITY).
        
        This is *not* a total ordering and is *not* suitable for use in Comparable.compareTo implementations. In particular, it is not transitive.

        Raises
        - IllegalArgumentException: if `tolerance` is `< 0` or NaN

        Since
        - 13.0
        """
        ...


    @staticmethod
    def mean(*values: Tuple[float, ...]) -> float:
        """
        Returns the <a href="http://en.wikipedia.org/wiki/Arithmetic_mean">arithmetic mean</a> of
        `values`.
        
        If these values are a sample drawn from a population, this is also an unbiased estimator of
        the arithmetic mean of the population.

        Arguments
        - values: a nonempty series of values

        Raises
        - IllegalArgumentException: if `values` is empty or contains any non-finite value

        Deprecated
        - Use Stats.meanOf instead, noting the less strict handling of non-finite
            values.
        """
        ...


    @staticmethod
    def mean(*values: Tuple[int, ...]) -> float:
        """
        Returns the <a href="http://en.wikipedia.org/wiki/Arithmetic_mean">arithmetic mean</a> of
        `values`.
        
        If these values are a sample drawn from a population, this is also an unbiased estimator of
        the arithmetic mean of the population.

        Arguments
        - values: a nonempty series of values

        Raises
        - IllegalArgumentException: if `values` is empty

        Deprecated
        - Use Stats.meanOf instead, noting the less strict handling of non-finite
            values.
        """
        ...


    @staticmethod
    def mean(*values: Tuple[int, ...]) -> float:
        """
        Returns the <a href="http://en.wikipedia.org/wiki/Arithmetic_mean">arithmetic mean</a> of
        `values`.
        
        If these values are a sample drawn from a population, this is also an unbiased estimator of
        the arithmetic mean of the population.

        Arguments
        - values: a nonempty series of values, which will be converted to `double` values
            (this may cause loss of precision for longs of magnitude over 2^53 (slightly over 9e15))

        Raises
        - IllegalArgumentException: if `values` is empty

        Deprecated
        - Use Stats.meanOf instead, noting the less strict handling of non-finite
            values.
        """
        ...


    @staticmethod
    def mean(values: Iterable["Number"]) -> float:
        """
        Returns the <a href="http://en.wikipedia.org/wiki/Arithmetic_mean">arithmetic mean</a> of
        `values`.
        
        If these values are a sample drawn from a population, this is also an unbiased estimator of
        the arithmetic mean of the population.

        Arguments
        - values: a nonempty series of values, which will be converted to `double` values
            (this may cause loss of precision)

        Raises
        - IllegalArgumentException: if `values` is empty or contains any non-finite value

        Deprecated
        - Use Stats.meanOf instead, noting the less strict handling of non-finite
            values.
        """
        ...


    @staticmethod
    def mean(values: Iterator["Number"]) -> float:
        """
        Returns the <a href="http://en.wikipedia.org/wiki/Arithmetic_mean">arithmetic mean</a> of
        `values`.
        
        If these values are a sample drawn from a population, this is also an unbiased estimator of
        the arithmetic mean of the population.

        Arguments
        - values: a nonempty series of values, which will be converted to `double` values
            (this may cause loss of precision)

        Raises
        - IllegalArgumentException: if `values` is empty or contains any non-finite value

        Deprecated
        - Use Stats.meanOf instead, noting the less strict handling of non-finite
            values.
        """
        ...



"""
Python module generated from Java source file com.google.common.graph.ElementOrder

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import Beta
from com.google.common.base import MoreObjects
from com.google.common.base.MoreObjects import ToStringHelper
from com.google.common.base import Objects
from com.google.common.collect import Maps
from com.google.common.collect import Ordering
from com.google.common.graph import *
from com.google.errorprone.annotations import Immutable
from enum import Enum
from java.util import Comparator
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class ElementOrder:
    """
    Used to represent the order of elements in a data structure that supports different options for
    iteration order guarantees.
    
    Example usage:
    
    ````MutableGraph<Integer> graph =
        GraphBuilder.directed().nodeOrder(ElementOrder.<Integer>natural()).build();````

    Author(s)
    - Joshua O'Madadhain

    Since
    - 20.0
    """

    @staticmethod
    def unordered() -> "ElementOrder"["S"]:
        """
        Returns an instance which specifies that no ordering is guaranteed.
        """
        ...


    @staticmethod
    def stable() -> "ElementOrder"["S"]:
        """
        Returns an instance which specifies that ordering is guaranteed to be always be the same across
        iterations, and across releases. Some methods may have stronger guarantees.
        
        This instance is only useful in combination with `incidentEdgeOrder`, e.g. `graphBuilder.incidentEdgeOrder(ElementOrder.stable())`.
        
        <h3>In combination with `incidentEdgeOrder`</h3>
        
        `incidentEdgeOrder(ElementOrder.stable())` guarantees the ordering of the returned
        collections of the following methods:
        
        
          - For Graph and ValueGraph:
              
                - `edges()`: Stable order
                - `adjacentNodes(node)`: Connecting edge insertion order
                - `predecessors(node)`: Connecting edge insertion order
                - `successors(node)`: Connecting edge insertion order
                - `incidentEdges(node)`: Edge insertion order
              
          - For Network:
              
                - `adjacentNodes(node)`: Stable order
                - `predecessors(node)`: Connecting edge insertion order
                - `successors(node)`: Connecting edge insertion order
                - `incidentEdges(node)`: Stable order
                - `inEdges(node)`: Edge insertion order
                - `outEdges(node)`: Edge insertion order
                - `adjacentEdges(edge)`: Stable order
                - `edgesConnecting(nodeU, nodeV)`: Edge insertion order
              

        Since
        - 29.0
        """
        ...


    @staticmethod
    def insertion() -> "ElementOrder"["S"]:
        """
        Returns an instance which specifies that insertion ordering is guaranteed.
        """
        ...


    @staticmethod
    def natural() -> "ElementOrder"["S"]:
        """
        Returns an instance which specifies that the natural ordering of the elements is guaranteed.
        """
        ...


    @staticmethod
    def sorted(comparator: "Comparator"["S"]) -> "ElementOrder"["S"]:
        """
        Returns an instance which specifies that the ordering of the elements is guaranteed to be
        determined by `comparator`.
        """
        ...


    def type(self) -> "Type":
        """
        Returns the type of ordering used.
        """
        ...


    def comparator(self) -> "Comparator"["T"]:
        """
        Returns the Comparator used.

        Raises
        - UnsupportedOperationException: if comparator is not defined
        """
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...


    def toString(self) -> str:
        ...


    class Type(Enum):
        """
        The type of ordering that this object specifies.
        
        
          - UNORDERED: no order is guaranteed.
          - STABLE: ordering is guaranteed to follow a pattern that won't change between releases.
              Some methods may have stronger guarantees.
          - INSERTION: insertion ordering is guaranteed.
          - SORTED: ordering according to a supplied comparator is guaranteed.
        """

        UNORDERED = 0
        STABLE = 1
        INSERTION = 2
        SORTED = 3



"""
Python module generated from Java source file org.bukkit.event.inventory.BrewingStandFuelEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import Block
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.block import BlockEvent
from org.bukkit.event.inventory import *
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class BrewingStandFuelEvent(BlockEvent, Cancellable):
    """
    Called when an ItemStack is about to increase the fuel level of a brewing
    stand.
    """

    def __init__(self, brewingStand: "Block", fuel: "ItemStack", fuelPower: int):
        ...


    def getFuel(self) -> "ItemStack":
        """
        Gets the ItemStack of the fuel before the amount was subtracted.

        Returns
        - the fuel ItemStack
        """
        ...


    def getFuelPower(self) -> int:
        """
        Gets the fuel power for this fuel. Each unit of power can fuel one
        brewing operation.

        Returns
        - the fuel power for this fuel
        """
        ...


    def setFuelPower(self, fuelPower: int) -> None:
        """
        Sets the fuel power for this fuel. Each unit of power can fuel one
        brewing operation.

        Arguments
        - fuelPower: the fuel power for this fuel
        """
        ...


    def isConsuming(self) -> bool:
        """
        Gets whether the brewing stand's fuel will be reduced / consumed or not.

        Returns
        - whether the fuel will be reduced or not
        """
        ...


    def setConsuming(self, consuming: bool) -> None:
        """
        Sets whether the brewing stand's fuel will be reduced / consumed or not.

        Arguments
        - consuming: whether the fuel will be reduced or not
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.ClosingFuture

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import J2ktIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.collect import FluentIterable
from com.google.common.collect import ImmutableList
from com.google.common.util.concurrent import *
from com.google.common.util.concurrent.ClosingFuture.Combiner import AsyncCombiningCallable
from com.google.common.util.concurrent.ClosingFuture.Combiner import CombiningCallable
from com.google.common.util.concurrent.Futures import FutureCombiner
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import DoNotMock
from com.google.j2objc.annotations import RetainedWith
from java.io import Closeable
from java.util import IdentityHashMap
from java.util.concurrent import Callable
from java.util.concurrent import CancellationException
from java.util.concurrent import CountDownLatch
from java.util.concurrent import ExecutionException
from java.util.concurrent import Executor
from java.util.concurrent import Future
from java.util.concurrent import RejectedExecutionException
from java.util.concurrent.atomic import AtomicReference
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ClosingFuture:

    @staticmethod
    def submit(callable: "ClosingCallable"["V"], executor: "Executor") -> "ClosingFuture"["V"]:
        """
        Starts a ClosingFuture pipeline by submitting a callable block to an executor.

        Raises
        - java.util.concurrent.RejectedExecutionException: if the task cannot be scheduled for
            execution
        """
        ...


    @staticmethod
    def submitAsync(callable: "AsyncClosingCallable"["V"], executor: "Executor") -> "ClosingFuture"["V"]:
        """
        Starts a ClosingFuture pipeline by submitting a callable block to an executor.

        Raises
        - java.util.concurrent.RejectedExecutionException: if the task cannot be scheduled for
            execution

        Since
        - 30.1
        """
        ...


    @staticmethod
    def from(future: "ListenableFuture"["V"]) -> "ClosingFuture"["V"]:
        """
        Starts a ClosingFuture pipeline with a ListenableFuture.
        
        `future`'s value will not be closed when the pipeline is done even if `V`
        implements Closeable. In order to start a pipeline with a value that will be closed
        when the pipeline is done, use .submit(ClosingCallable, Executor) instead.
        """
        ...


    @staticmethod
    def eventuallyClosing(future: "ListenableFuture"["C"], closingExecutor: "Executor") -> "ClosingFuture"["C"]:
        """
        Starts a ClosingFuture pipeline with a ListenableFuture.
        
        If `future` succeeds, its value will be closed (using `closingExecutor)`) when
        the pipeline is done, even if the pipeline is canceled or fails.
        
        Cancelling the pipeline will not cancel `future`, so that the pipeline can access its
        value in order to close it.

        Arguments
        - future: the future to create the `ClosingFuture` from. For discussion of the
            future's result type `C`, see DeferredCloser.eventuallyClose(Object,
            Executor).
        - closingExecutor: the future's result will be closed on this executor

        Deprecated
        - Creating Futures of closeable types is dangerous in general because the
            underlying value may never be closed if the Future is canceled after its operation
            begins. Consider replacing code that creates ListenableFutures of closeable types,
            including those that pass them to this method, with .submit(ClosingCallable,
            Executor) in order to ensure that resources do not leak. Or, to start a pipeline with a
            ListenableFuture that doesn't create values that should be closed, use ClosingFuture.from.
        """
        ...


    @staticmethod
    def whenAllComplete(futures: Iterable["ClosingFuture"[Any]]) -> "Combiner":
        """
        Starts specifying how to combine ClosingFutures into a single pipeline.

        Raises
        - IllegalStateException: if a `ClosingFuture` has already been derived from any of
            the `futures`, or if any has already been .finishToFuture() finished
        """
        ...


    @staticmethod
    def whenAllComplete(future1: "ClosingFuture"[Any], *moreFutures: Tuple["ClosingFuture"[Any], ...]) -> "Combiner":
        """
        Starts specifying how to combine ClosingFutures into a single pipeline.

        Raises
        - IllegalStateException: if a `ClosingFuture` has already been derived from any of
            the arguments, or if any has already been .finishToFuture() finished
        """
        ...


    @staticmethod
    def whenAllSucceed(futures: Iterable["ClosingFuture"[Any]]) -> "Combiner":
        """
        Starts specifying how to combine ClosingFutures into a single pipeline, assuming they
        all succeed. If any fail, the resulting pipeline will fail.

        Raises
        - IllegalStateException: if a `ClosingFuture` has already been derived from any of
            the `futures`, or if any has already been .finishToFuture() finished
        """
        ...


    @staticmethod
    def whenAllSucceed(future1: "ClosingFuture"["V1"], future2: "ClosingFuture"["V2"]) -> "Combiner2"["V1", "V2"]:
        """
        Starts specifying how to combine two ClosingFutures into a single pipeline, assuming
        they all succeed. If any fail, the resulting pipeline will fail.
        
        Calling this method allows you to use lambdas or method references typed with the types of
        the input ClosingFutures.

        Raises
        - IllegalStateException: if a `ClosingFuture` has already been derived from any of
            the arguments, or if any has already been .finishToFuture() finished
        """
        ...


    @staticmethod
    def whenAllSucceed(future1: "ClosingFuture"["V1"], future2: "ClosingFuture"["V2"], future3: "ClosingFuture"["V3"]) -> "Combiner3"["V1", "V2", "V3"]:
        """
        Starts specifying how to combine three ClosingFutures into a single pipeline, assuming
        they all succeed. If any fail, the resulting pipeline will fail.
        
        Calling this method allows you to use lambdas or method references typed with the types of
        the input ClosingFutures.

        Raises
        - IllegalStateException: if a `ClosingFuture` has already been derived from any of
            the arguments, or if any has already been .finishToFuture() finished
        """
        ...


    @staticmethod
    def whenAllSucceed(future1: "ClosingFuture"["V1"], future2: "ClosingFuture"["V2"], future3: "ClosingFuture"["V3"], future4: "ClosingFuture"["V4"]) -> "Combiner4"["V1", "V2", "V3", "V4"]:
        """
        Starts specifying how to combine four ClosingFutures into a single pipeline, assuming
        they all succeed. If any fail, the resulting pipeline will fail.
        
        Calling this method allows you to use lambdas or method references typed with the types of
        the input ClosingFutures.

        Raises
        - IllegalStateException: if a `ClosingFuture` has already been derived from any of
            the arguments, or if any has already been .finishToFuture() finished
        """
        ...


    @staticmethod
    def whenAllSucceed(future1: "ClosingFuture"["V1"], future2: "ClosingFuture"["V2"], future3: "ClosingFuture"["V3"], future4: "ClosingFuture"["V4"], future5: "ClosingFuture"["V5"]) -> "Combiner5"["V1", "V2", "V3", "V4", "V5"]:
        """
        Starts specifying how to combine five ClosingFutures into a single pipeline, assuming
        they all succeed. If any fail, the resulting pipeline will fail.
        
        Calling this method allows you to use lambdas or method references typed with the types of
        the input ClosingFutures.

        Raises
        - IllegalStateException: if a `ClosingFuture` has already been derived from any of
            the arguments, or if any has already been .finishToFuture() finished
        """
        ...


    @staticmethod
    def whenAllSucceed(future1: "ClosingFuture"[Any], future2: "ClosingFuture"[Any], future3: "ClosingFuture"[Any], future4: "ClosingFuture"[Any], future5: "ClosingFuture"[Any], future6: "ClosingFuture"[Any], *moreFutures: Tuple["ClosingFuture"[Any], ...]) -> "Combiner":
        """
        Starts specifying how to combine ClosingFutures into a single pipeline, assuming they
        all succeed. If any fail, the resulting pipeline will fail.

        Raises
        - IllegalStateException: if a `ClosingFuture` has already been derived from any of
            the arguments, or if any has already been .finishToFuture() finished
        """
        ...


    def statusFuture(self) -> "ListenableFuture"[Any]:
        """
        Returns a future that finishes when this step does. Calling `get()` on the returned
        future returns `null` if the step is successful or throws the same exception that would
        be thrown by calling `finishToFuture().get()` if this were the last step. Calling `cancel()` on the returned future has no effect on the `ClosingFuture` pipeline.
        
        `statusFuture` differs from most methods on `ClosingFuture`: You can make calls
        to `statusFuture` *in addition to* the call you make to .finishToFuture() or
        a derivation method *on the same instance*. This is important because calling `statusFuture` alone does not provide a way to close the pipeline.
        """
        ...


    def transform(self, function: "ClosingFunction"["V", "U"], executor: "Executor") -> "ClosingFuture"["U"]:
        """
        Returns a new `ClosingFuture` pipeline step derived from this one by applying a function
        to its value. The function can use a DeferredCloser to capture objects to be closed
        when the pipeline is done.
        
        If this `ClosingFuture` fails, the function will not be called, and the derived `ClosingFuture` will be equivalent to this one.
        
        If the function throws an exception, that exception is used as the result of the derived
        `ClosingFuture`.
        
        Example usage:
        
        ````ClosingFuture<List<Row>> rowsFuture =
            queryFuture.transform((closer, result) -> result.getRows(), executor);````
        
        When selecting an executor, note that `directExecutor` is dangerous in some cases. See
        the discussion in the ListenableFuture.addListener documentation. All its warnings
        about heavyweight listeners are also applicable to heavyweight functions passed to this method.
        
        After calling this method, you may not call .finishToFuture(), .finishToValueAndCloser(ValueAndCloserConsumer, Executor), or any other derivation method on
        the original `ClosingFuture` instance.

        Arguments
        - function: transforms the value of this step to the value of the derived step
        - executor: executor to run the function in

        Returns
        - the derived step

        Raises
        - IllegalStateException: if a `ClosingFuture` has already been derived from this
            one, or if this `ClosingFuture` has already been .finishToFuture()
            finished
        """
        ...


    def transformAsync(self, function: "AsyncClosingFunction"["V", "U"], executor: "Executor") -> "ClosingFuture"["U"]:
        """
        Returns a new `ClosingFuture` pipeline step derived from this one by applying a function
        that returns a `ClosingFuture` to its value. The function can use a DeferredCloser to capture objects to be closed when the pipeline is done (other than those
        captured by the returned ClosingFuture).
        
        If this `ClosingFuture` succeeds, the derived one will be equivalent to the one
        returned by the function.
        
        If this `ClosingFuture` fails, the function will not be called, and the derived `ClosingFuture` will be equivalent to this one.
        
        If the function throws an exception, that exception is used as the result of the derived
        `ClosingFuture`. But if the exception is thrown after the function creates a `ClosingFuture`, then none of the closeable objects in that `ClosingFuture` will be
        closed.
        
        Usage guidelines for this method:
        
        
          - Use this method only when calling an API that returns a ListenableFuture or a
              `ClosingFuture`. If possible, prefer calling .transform(ClosingFunction,
              Executor) instead, with a function that returns the next value directly.
          - Call DeferredCloser.eventuallyClose(Object, Executor) closer.eventuallyClose()
              for every closeable object this step creates in order to capture it for later closing.
          - Return a `ClosingFuture`. To turn a ListenableFuture into a `ClosingFuture` call .from(ListenableFuture).
          - In case this step doesn't create new closeables, you can adapt an API that returns a
              ListenableFuture to return a `ClosingFuture` by wrapping it with a call to
              .withoutCloser(AsyncFunction)
        
        
        Example usage:
        
        ````// Result.getRowsClosingFuture() returns a ClosingFuture.
        ClosingFuture<List<Row>> rowsFuture =
            queryFuture.transformAsync((closer, result) -> result.getRowsClosingFuture(), executor);
        
        // Result.writeRowsToOutputStreamFuture() returns a ListenableFuture that resolves to the
        // number of written rows. openOutputFile() returns a FileOutputStream (which implements
        // Closeable).
        ClosingFuture<Integer> rowsFuture2 =
            queryFuture.transformAsync(
                (closer, result) -> {
                  FileOutputStream fos = closer.eventuallyClose(openOutputFile(), closingExecutor);
                  return ClosingFuture.from(result.writeRowsToOutputStreamFuture(fos));`,
             executor);
        
        // Result.getRowsFuture() returns a ListenableFuture (no new closeables are created).
        ClosingFuture<List<Row>> rowsFuture3 =
            queryFuture.transformAsync(withoutCloser(Result::getRowsFuture), executor);
        
        }```
        
        When selecting an executor, note that `directExecutor` is dangerous in some cases. See
        the discussion in the ListenableFuture.addListener documentation. All its warnings
        about heavyweight listeners are also applicable to heavyweight functions passed to this method.
        (Specifically, `directExecutor` functions should avoid heavyweight operations inside
        `AsyncClosingFunction.apply`. Any heavyweight operations should occur in other threads
        responsible for completing the returned `ClosingFuture`.)
        
        After calling this method, you may not call .finishToFuture(), .finishToValueAndCloser(ValueAndCloserConsumer, Executor), or any other derivation method on
        the original `ClosingFuture` instance.

        Arguments
        - function: transforms the value of this step to a `ClosingFuture` with the value of
            the derived step
        - executor: executor to run the function in

        Returns
        - the derived step

        Raises
        - IllegalStateException: if a `ClosingFuture` has already been derived from this
            one, or if this `ClosingFuture` has already been .finishToFuture()
            finished
        """
        ...


    @staticmethod
    def withoutCloser(function: "AsyncFunction"["V", "U"]) -> "AsyncClosingFunction"["V", "U"]:
        """
        Returns an AsyncClosingFunction that applies an AsyncFunction to an input,
        ignoring the DeferredCloser and returning a `ClosingFuture` derived from the returned
        ListenableFuture.
        
        Use this method to pass a transformation to .transformAsync(AsyncClosingFunction,
        Executor) or to .catchingAsync(Class, AsyncClosingFunction, Executor) as long as it
        meets these conditions:
        
        
          - It does not need to capture any Closeable objects by calling DeferredCloser.eventuallyClose(Object, Executor).
          - It returns a ListenableFuture.
        
        
        Example usage:
        
        ````// Result.getRowsFuture() returns a ListenableFuture.
        ClosingFuture<List<Row>> rowsFuture =
            queryFuture.transformAsync(withoutCloser(Result::getRowsFuture), executor);````

        Arguments
        - function: transforms the value of a `ClosingFuture` step to a ListenableFuture with the value of a derived step
        """
        ...


    def catching(self, exceptionType: type["X"], fallback: "ClosingFunction"["X", "V"], executor: "Executor") -> "ClosingFuture"["V"]:
        """
        Returns a new `ClosingFuture` pipeline step derived from this one by applying a function
        to its exception if it is an instance of a given exception type. The function can use a DeferredCloser to capture objects to be closed when the pipeline is done.
        
        If this `ClosingFuture` succeeds or fails with a different exception type, the
        function will not be called, and the derived `ClosingFuture` will be equivalent to this
        one.
        
        If the function throws an exception, that exception is used as the result of the derived
        `ClosingFuture`.
        
        Example usage:
        
        ````ClosingFuture<QueryResult> queryFuture =
            queryFuture.catching(
                QueryException.class, (closer, x) -> Query.emptyQueryResult(), executor);````
        
        When selecting an executor, note that `directExecutor` is dangerous in some cases. See
        the discussion in the ListenableFuture.addListener documentation. All its warnings
        about heavyweight listeners are also applicable to heavyweight functions passed to this method.
        
        After calling this method, you may not call .finishToFuture(), .finishToValueAndCloser(ValueAndCloserConsumer, Executor), or any other derivation method on
        the original `ClosingFuture` instance.

        Arguments
        - exceptionType: the exception type that triggers use of `fallback`. The exception
            type is matched against this step's exception. "This step's exception" means the cause of
            the ExecutionException thrown by Future.get() on the Future
            underlying this step or, if `get()` throws a different kind of exception, that
            exception itself. To avoid hiding bugs and other unrecoverable errors, callers should
            prefer more specific types, avoiding `Throwable.class` in particular.
        - fallback: the function to be called if this step fails with the expected exception type.
            The function's argument is this step's exception. "This step's exception" means the cause
            of the ExecutionException thrown by Future.get() on the Future
            underlying this step or, if `get()` throws a different kind of exception, that
            exception itself.
        - executor: the executor that runs `fallback` if the input fails
        """
        ...


    def catchingAsync(self, exceptionType: type["X"], fallback: "AsyncClosingFunction"["X", "V"], executor: "Executor") -> "ClosingFuture"["V"]:
        ...


    def finishToFuture(self) -> "FluentFuture"["V"]:
        """
        Marks this step as the last step in the `ClosingFuture` pipeline.
        
        The returned Future is completed when the pipeline's computation completes, or when
        the pipeline is cancelled.
        
        All objects the pipeline has captured for closing will begin to be closed asynchronously
        **after** the returned `Future` is done: the future completes before closing starts,
        rather than once it has finished.
        
        After calling this method, you may not call .finishToValueAndCloser(ValueAndCloserConsumer, Executor), this method, or any other
        derivation method on the original `ClosingFuture` instance.

        Returns
        - a Future that represents the final value or exception of the pipeline
        """
        ...


    def finishToValueAndCloser(self, consumer: "ValueAndCloserConsumer"["V"], executor: "Executor") -> None:
        """
        Marks this step as the last step in the `ClosingFuture` pipeline. When this step is done,
        `receiver` will be called with an object that contains the result of the operation. The
        receiver can store the ValueAndCloser outside the receiver for later synchronous use.
        
        After calling this method, you may not call .finishToFuture(), this method again, or
        any other derivation method on the original `ClosingFuture` instance.

        Arguments
        - consumer: a callback whose method will be called (using `executor`) when this
            operation is done
        """
        ...


    def cancel(self, mayInterruptIfRunning: bool) -> bool:
        """
        Attempts to cancel execution of this step. This attempt will fail if the step has already
        completed, has already been cancelled, or could not be cancelled for some other reason. If
        successful, and this step has not started when `cancel` is called, this step should never
        run.
        
        If successful, causes the objects captured by this step (if already started) and its input
        step(s) for later closing to be closed on their respective Executors. If any such calls
        specified MoreExecutors.directExecutor(), those objects will be closed synchronously.

        Arguments
        - mayInterruptIfRunning: `True` if the thread executing this task should be
            interrupted; otherwise, in-progress tasks are allowed to complete, but the step will be
            cancelled regardless

        Returns
        - `False` if the step could not be cancelled, typically because it has already
            completed normally; `True` otherwise
        """
        ...


    def toString(self) -> str:
        ...


    class DeferredCloser:
        """
        An object that can capture objects to be closed later, when a ClosingFuture pipeline is
        done.
        """

        def eventuallyClose(self, closeable: "C", closingExecutor: "Executor") -> "C":
            """
            Captures an object to be closed when a ClosingFuture pipeline is done.
            
            For users of the `-jre` flavor of Guava, the object can be any `AutoCloseable`. For users of the `-android` flavor, the object must be a `Closeable`. (For more about the flavors, see <a
            href="https://github.com/google/guava#adding-guava-to-your-build">Adding Guava to your
            build</a>.)
            
            Be careful when targeting an older SDK than you are building against (most commonly when
            building for Android): Ensure that any object you pass implements the interface not just in
            your current SDK version but also at the oldest version you support. For example, <a
            href="https://developer.android.com/sdk/api_diff/16/">API Level 16</a> is the first version
            in which `Cursor` is `Closeable`. To support older versions, pass a wrapper
            `Closeable` with a method reference like `cursor::close`.
            
            Note that this method is still binary-compatible between flavors because the erasure of
            its parameter type is `Object`, not `AutoCloseable` or `Closeable`.

            Arguments
            - closeable: the object to be closed (see notes above)
            - closingExecutor: the object will be closed on this executor

            Returns
            - the first argument
            """
            ...


    class ClosingCallable:
        """
        An operation that computes a result.
        
        Type `<V>`: the type of the result
        """

        def call(self, closer: "DeferredCloser") -> "V":
            """
            Computes a result, or throws an exception if unable to do so.
            
            Any objects that are passed to DeferredCloser.eventuallyClose(Object, Executor)
            closer.eventuallyClose() will be closed when the ClosingFuture pipeline is done (but
            not before this method completes), even if this method throws or the pipeline is cancelled.
            """
            ...


    class AsyncClosingCallable:
        """
        An operation that computes a ClosingFuture of a result.
        
        Type `<V>`: the type of the result

        Since
        - 30.1
        """

        def call(self, closer: "DeferredCloser") -> "ClosingFuture"["V"]:
            """
            Computes a result, or throws an exception if unable to do so.
            
            Any objects that are passed to DeferredCloser.eventuallyClose(Object, Executor)
            closer.eventuallyClose() will be closed when the ClosingFuture pipeline is done (but
            not before this method completes), even if this method throws or the pipeline is cancelled.
            """
            ...


    class ClosingFunction:
        """
        A function from an input to a result.
        
        Type `<T>`: the type of the input to the function
        
        Type `<U>`: the type of the result of the function
        """

        def apply(self, closer: "DeferredCloser", input: "T") -> "U":
            """
            Applies this function to an input, or throws an exception if unable to do so.
            
            Any objects that are passed to DeferredCloser.eventuallyClose(Object, Executor)
            closer.eventuallyClose() will be closed when the ClosingFuture pipeline is done (but
            not before this method completes), even if this method throws or the pipeline is cancelled.
            """
            ...


    class AsyncClosingFunction:
        """
        A function from an input to a ClosingFuture of a result.
        
        Type `<T>`: the type of the input to the function
        
        Type `<U>`: the type of the result of the function
        """

        def apply(self, closer: "DeferredCloser", input: "T") -> "ClosingFuture"["U"]:
            """
            Applies this function to an input, or throws an exception if unable to do so.
            
            Any objects that are passed to DeferredCloser.eventuallyClose(Object, Executor)
            closer.eventuallyClose() will be closed when the ClosingFuture pipeline is done (but
            not before this method completes), even if this method throws or the pipeline is cancelled.
            """
            ...


    class ValueAndCloser:
        """
        An object that holds the final result of an asynchronous ClosingFuture operation and
        allows the user to close all the closeable objects that were captured during it for later
        closing.
        
        The asynchronous operation will have completed before this object is created.
        
        Type `<V>`: the type of the value of a successful operation

        See
        - ClosingFuture.finishToValueAndCloser(ValueAndCloserConsumer, Executor)
        """

        def get(self) -> "V":
            """
            Returns the final value of the associated ClosingFuture, or throws an exception as
            Future.get() would.
            
            Because the asynchronous operation has already completed, this method is synchronous and
            returns immediately.

            Raises
            - CancellationException: if the computation was cancelled
            - ExecutionException: if the computation threw an exception
            """
            ...


        def closeAsync(self) -> None:
            """
            Starts closing all closeable objects captured during the ClosingFuture's asynchronous
            operation on the Executors specified by calls to DeferredCloser.eventuallyClose(Object, Executor).
            
            If any such calls specified MoreExecutors.directExecutor(), those objects will be
            closed synchronously.
            
            Idempotent: objects will be closed at most once.
            """
            ...


    class ValueAndCloserConsumer:
        """
        Represents an operation that accepts a ValueAndCloser for the last step in a ClosingFuture pipeline.
        
        Type `<V>`: the type of the final value of a successful pipeline

        See
        - ClosingFuture.finishToValueAndCloser(ValueAndCloserConsumer, Executor)
        """

        def accept(self, valueAndCloser: "ValueAndCloser"["V"]) -> None:
            """
            Accepts a ValueAndCloser for the last step in a ClosingFuture pipeline.
            """
            ...


    class Peeker:
        """
        An object that can return the value of the ClosingFutures that are passed to .whenAllComplete(Iterable) or .whenAllSucceed(Iterable).
        
        Only for use by a CombiningCallable or AsyncCombiningCallable object.
        """

        def getDone(self, closingFuture: "ClosingFuture"["D"]) -> "D":
            """
            Returns the value of `closingFuture`.

            Raises
            - ExecutionException: if `closingFuture` is a failed step
            - CancellationException: if the `closingFuture`'s future was cancelled
            - IllegalArgumentException: if `closingFuture` is not one of the futures passed to
                .whenAllComplete(Iterable) or .whenAllComplete(Iterable)
            - IllegalStateException: if called outside of a call to CombiningCallable.call(DeferredCloser, Peeker) or AsyncCombiningCallable.call(DeferredCloser, Peeker)
            """
            ...


    class Combiner:

        def call(self, combiningCallable: "CombiningCallable"["V"], executor: "Executor") -> "ClosingFuture"["V"]:
            """
            Returns a new `ClosingFuture` pipeline step derived from the inputs by applying a
            combining function to their values. The function can use a DeferredCloser to capture
            objects to be closed when the pipeline is done.
            
            If this combiner was returned by a .whenAllSucceed method and any of the inputs
            fail, so will the returned step.
            
            If the combiningCallable throws a `CancellationException`, the pipeline will be
            cancelled.
            
            If the combiningCallable throws an `ExecutionException`, the cause of the thrown
            `ExecutionException` will be extracted and used as the failure of the derived step.
            """
            ...


        def callAsync(self, combiningCallable: "AsyncCombiningCallable"["V"], executor: "Executor") -> "ClosingFuture"["V"]:
            """
            Returns a new `ClosingFuture` pipeline step derived from the inputs by applying a
            `ClosingFuture`-returning function to their values. The function can use a DeferredCloser to capture objects to be closed when the pipeline is done (other than those
            captured by the returned ClosingFuture).
            
            If this combiner was returned by a .whenAllSucceed method and any of the inputs
            fail, so will the returned step.
            
            If the combiningCallable throws a `CancellationException`, the pipeline will be
            cancelled.
            
            If the combiningCallable throws an `ExecutionException`, the cause of the thrown
            `ExecutionException` will be extracted and used as the failure of the derived step.
            
            If the combiningCallable throws any other exception, it will be used as the failure of the
            derived step.
            
            If an exception is thrown after the combiningCallable creates a `ClosingFuture`,
            then none of the closeable objects in that `ClosingFuture` will be closed.
            
            Usage guidelines for this method:
            
            
              - Use this method only when calling an API that returns a ListenableFuture or a
                  `ClosingFuture`. If possible, prefer calling .call(CombiningCallable,
                  Executor) instead, with a function that returns the next value directly.
              - Call DeferredCloser.eventuallyClose(Object, Executor) closer.eventuallyClose()
                  for every closeable object this step creates in order to capture it for later closing.
              - Return a `ClosingFuture`. To turn a ListenableFuture into a `ClosingFuture` call .from(ListenableFuture).
            
            
            The same warnings about doing heavyweight operations within ClosingFuture.transformAsync(AsyncClosingFunction, Executor) apply here.
            """
            ...


        class CombiningCallable:
            """
            An operation that returns a result and may throw an exception.
            
            Type `<V>`: the type of the result
            """

            def call(self, closer: "DeferredCloser", peeker: "Peeker") -> "V":
                """
                Computes a result, or throws an exception if unable to do so.
                
                Any objects that are passed to DeferredCloser.eventuallyClose(Object, Executor)
                closer.eventuallyClose() will be closed when the ClosingFuture pipeline is done
                (but not before this method completes), even if this method throws or the pipeline is
                cancelled.

                Arguments
                - peeker: used to get the value of any of the input futures
                """
                ...


        class AsyncCombiningCallable:
            """
            An operation that returns a ClosingFuture result and may throw an exception.
            
            Type `<V>`: the type of the result
            """

            def call(self, closer: "DeferredCloser", peeker: "Peeker") -> "ClosingFuture"["V"]:
                """
                Computes a ClosingFuture result, or throws an exception if unable to do so.
                
                Any objects that are passed to DeferredCloser.eventuallyClose(Object, Executor)
                closer.eventuallyClose() will be closed when the ClosingFuture pipeline is done
                (but not before this method completes), even if this method throws or the pipeline is
                cancelled.

                Arguments
                - peeker: used to get the value of any of the input futures
                """
                ...


    class Combiner2(Combiner):
        """
        A generic Combiner that lets you use a lambda or method reference to combine two ClosingFutures. Use .whenAllSucceed(ClosingFuture, ClosingFuture) to start this
        combination.

        Arguments
        - <V1>: the type returned by the first future
        - <V2>: the type returned by the second future
        """

        def call(self, function: "ClosingFunction2"["V1", "V2", "U"], executor: "Executor") -> "ClosingFuture"["U"]:
            """
            Returns a new `ClosingFuture` pipeline step derived from the inputs by applying a
            combining function to their values. The function can use a DeferredCloser to capture
            objects to be closed when the pipeline is done.
            
            If this combiner was returned by .whenAllSucceed(ClosingFuture, ClosingFuture) and
            any of the inputs fail, so will the returned step.
            
            If the function throws a `CancellationException`, the pipeline will be cancelled.
            
            If the function throws an `ExecutionException`, the cause of the thrown `ExecutionException` will be extracted and used as the failure of the derived step.
            """
            ...


        def callAsync(self, function: "AsyncClosingFunction2"["V1", "V2", "U"], executor: "Executor") -> "ClosingFuture"["U"]:
            """
            Returns a new `ClosingFuture` pipeline step derived from the inputs by applying a
            `ClosingFuture`-returning function to their values. The function can use a DeferredCloser to capture objects to be closed when the pipeline is done (other than those
            captured by the returned ClosingFuture).
            
            If this combiner was returned by .whenAllSucceed(ClosingFuture, ClosingFuture) and
            any of the inputs fail, so will the returned step.
            
            If the function throws a `CancellationException`, the pipeline will be cancelled.
            
            If the function throws an `ExecutionException`, the cause of the thrown `ExecutionException` will be extracted and used as the failure of the derived step.
            
            If the function throws any other exception, it will be used as the failure of the derived
            step.
            
            If an exception is thrown after the function creates a `ClosingFuture`, then none of
            the closeable objects in that `ClosingFuture` will be closed.
            
            Usage guidelines for this method:
            
            
              - Use this method only when calling an API that returns a ListenableFuture or a
                  `ClosingFuture`. If possible, prefer calling .call(CombiningCallable,
                  Executor) instead, with a function that returns the next value directly.
              - Call DeferredCloser.eventuallyClose(Object, Executor) closer.eventuallyClose()
                  for every closeable object this step creates in order to capture it for later closing.
              - Return a `ClosingFuture`. To turn a ListenableFuture into a `ClosingFuture` call .from(ListenableFuture).
            
            
            The same warnings about doing heavyweight operations within ClosingFuture.transformAsync(AsyncClosingFunction, Executor) apply here.
            """
            ...


        class ClosingFunction2:
            """
            A function that returns a value when applied to the values of the two futures passed to
            .whenAllSucceed(ClosingFuture, ClosingFuture).
            
            Type `<U>`: the type returned by the function

            Arguments
            - <V1>: the type returned by the first future
            - <V2>: the type returned by the second future
            """

            def apply(self, closer: "DeferredCloser", value1: "V1", value2: "V2") -> "U":
                """
                Applies this function to two inputs, or throws an exception if unable to do so.
                
                Any objects that are passed to DeferredCloser.eventuallyClose(Object, Executor)
                closer.eventuallyClose() will be closed when the ClosingFuture pipeline is done
                (but not before this method completes), even if this method throws or the pipeline is
                cancelled.
                """
                ...


        class AsyncClosingFunction2:
            """
            A function that returns a ClosingFuture when applied to the values of the two futures
            passed to .whenAllSucceed(ClosingFuture, ClosingFuture).
            
            Type `<U>`: the type returned by the function

            Arguments
            - <V1>: the type returned by the first future
            - <V2>: the type returned by the second future
            """

            def apply(self, closer: "DeferredCloser", value1: "V1", value2: "V2") -> "ClosingFuture"["U"]:
                """
                Applies this function to two inputs, or throws an exception if unable to do so.
                
                Any objects that are passed to DeferredCloser.eventuallyClose(Object, Executor)
                closer.eventuallyClose() will be closed when the ClosingFuture pipeline is done
                (but not before this method completes), even if this method throws or the pipeline is
                cancelled.
                """
                ...


    class Combiner3(Combiner):
        """
        A generic Combiner that lets you use a lambda or method reference to combine three
        ClosingFutures. Use .whenAllSucceed(ClosingFuture, ClosingFuture,
        ClosingFuture) to start this combination.

        Arguments
        - <V1>: the type returned by the first future
        - <V2>: the type returned by the second future
        - <V3>: the type returned by the third future
        """

        def call(self, function: "ClosingFunction3"["V1", "V2", "V3", "U"], executor: "Executor") -> "ClosingFuture"["U"]:
            """
            Returns a new `ClosingFuture` pipeline step derived from the inputs by applying a
            combining function to their values. The function can use a DeferredCloser to capture
            objects to be closed when the pipeline is done.
            
            If this combiner was returned by .whenAllSucceed(ClosingFuture, ClosingFuture,
            ClosingFuture) and any of the inputs fail, so will the returned step.
            
            If the function throws a `CancellationException`, the pipeline will be cancelled.
            
            If the function throws an `ExecutionException`, the cause of the thrown `ExecutionException` will be extracted and used as the failure of the derived step.
            """
            ...


        def callAsync(self, function: "AsyncClosingFunction3"["V1", "V2", "V3", "U"], executor: "Executor") -> "ClosingFuture"["U"]:
            """
            Returns a new `ClosingFuture` pipeline step derived from the inputs by applying a
            `ClosingFuture`-returning function to their values. The function can use a DeferredCloser to capture objects to be closed when the pipeline is done (other than those
            captured by the returned ClosingFuture).
            
            If this combiner was returned by .whenAllSucceed(ClosingFuture, ClosingFuture,
            ClosingFuture) and any of the inputs fail, so will the returned step.
            
            If the function throws a `CancellationException`, the pipeline will be cancelled.
            
            If the function throws an `ExecutionException`, the cause of the thrown `ExecutionException` will be extracted and used as the failure of the derived step.
            
            If the function throws any other exception, it will be used as the failure of the derived
            step.
            
            If an exception is thrown after the function creates a `ClosingFuture`, then none of
            the closeable objects in that `ClosingFuture` will be closed.
            
            Usage guidelines for this method:
            
            
              - Use this method only when calling an API that returns a ListenableFuture or a
                  `ClosingFuture`. If possible, prefer calling .call(CombiningCallable,
                  Executor) instead, with a function that returns the next value directly.
              - Call DeferredCloser.eventuallyClose(Object, Executor) closer.eventuallyClose()
                  for every closeable object this step creates in order to capture it for later closing.
              - Return a `ClosingFuture`. To turn a ListenableFuture into a `ClosingFuture` call .from(ListenableFuture).
            
            
            The same warnings about doing heavyweight operations within ClosingFuture.transformAsync(AsyncClosingFunction, Executor) apply here.
            """
            ...


        class ClosingFunction3:
            """
            A function that returns a value when applied to the values of the three futures passed to
            .whenAllSucceed(ClosingFuture, ClosingFuture, ClosingFuture).
            
            Type `<U>`: the type returned by the function

            Arguments
            - <V1>: the type returned by the first future
            - <V2>: the type returned by the second future
            - <V3>: the type returned by the third future
            """

            def apply(self, closer: "DeferredCloser", value1: "V1", value2: "V2", value3: "V3") -> "U":
                """
                Applies this function to three inputs, or throws an exception if unable to do so.
                
                Any objects that are passed to DeferredCloser.eventuallyClose(Object, Executor)
                closer.eventuallyClose() will be closed when the ClosingFuture pipeline is done
                (but not before this method completes), even if this method throws or the pipeline is
                cancelled.
                """
                ...


        class AsyncClosingFunction3:
            """
            A function that returns a ClosingFuture when applied to the values of the three
            futures passed to .whenAllSucceed(ClosingFuture, ClosingFuture, ClosingFuture).
            
            Type `<U>`: the type returned by the function

            Arguments
            - <V1>: the type returned by the first future
            - <V2>: the type returned by the second future
            - <V3>: the type returned by the third future
            """

            def apply(self, closer: "DeferredCloser", value1: "V1", value2: "V2", value3: "V3") -> "ClosingFuture"["U"]:
                """
                Applies this function to three inputs, or throws an exception if unable to do so.
                
                Any objects that are passed to DeferredCloser.eventuallyClose(Object, Executor)
                closer.eventuallyClose() will be closed when the ClosingFuture pipeline is done
                (but not before this method completes), even if this method throws or the pipeline is
                cancelled.
                """
                ...


    class Combiner4(Combiner):
        """
        A generic Combiner that lets you use a lambda or method reference to combine four
        ClosingFutures. Use .whenAllSucceed(ClosingFuture, ClosingFuture, ClosingFuture,
        ClosingFuture) to start this combination.

        Arguments
        - <V1>: the type returned by the first future
        - <V2>: the type returned by the second future
        - <V3>: the type returned by the third future
        - <V4>: the type returned by the fourth future
        """

        def call(self, function: "ClosingFunction4"["V1", "V2", "V3", "V4", "U"], executor: "Executor") -> "ClosingFuture"["U"]:
            """
            Returns a new `ClosingFuture` pipeline step derived from the inputs by applying a
            combining function to their values. The function can use a DeferredCloser to capture
            objects to be closed when the pipeline is done.
            
            If this combiner was returned by .whenAllSucceed(ClosingFuture, ClosingFuture,
            ClosingFuture, ClosingFuture) and any of the inputs fail, so will the returned step.
            
            If the function throws a `CancellationException`, the pipeline will be cancelled.
            
            If the function throws an `ExecutionException`, the cause of the thrown `ExecutionException` will be extracted and used as the failure of the derived step.
            """
            ...


        def callAsync(self, function: "AsyncClosingFunction4"["V1", "V2", "V3", "V4", "U"], executor: "Executor") -> "ClosingFuture"["U"]:
            """
            Returns a new `ClosingFuture` pipeline step derived from the inputs by applying a
            `ClosingFuture`-returning function to their values. The function can use a DeferredCloser to capture objects to be closed when the pipeline is done (other than those
            captured by the returned ClosingFuture).
            
            If this combiner was returned by .whenAllSucceed(ClosingFuture, ClosingFuture,
            ClosingFuture, ClosingFuture) and any of the inputs fail, so will the returned step.
            
            If the function throws a `CancellationException`, the pipeline will be cancelled.
            
            If the function throws an `ExecutionException`, the cause of the thrown `ExecutionException` will be extracted and used as the failure of the derived step.
            
            If the function throws any other exception, it will be used as the failure of the derived
            step.
            
            If an exception is thrown after the function creates a `ClosingFuture`, then none of
            the closeable objects in that `ClosingFuture` will be closed.
            
            Usage guidelines for this method:
            
            
              - Use this method only when calling an API that returns a ListenableFuture or a
                  `ClosingFuture`. If possible, prefer calling .call(CombiningCallable,
                  Executor) instead, with a function that returns the next value directly.
              - Call DeferredCloser.eventuallyClose(Object, Executor) closer.eventuallyClose()
                  for every closeable object this step creates in order to capture it for later closing.
              - Return a `ClosingFuture`. To turn a ListenableFuture into a `ClosingFuture` call .from(ListenableFuture).
            
            
            The same warnings about doing heavyweight operations within ClosingFuture.transformAsync(AsyncClosingFunction, Executor) apply here.
            """
            ...


        class ClosingFunction4:
            """
            A function that returns a value when applied to the values of the four futures passed to
            .whenAllSucceed(ClosingFuture, ClosingFuture, ClosingFuture, ClosingFuture).
            
            Type `<U>`: the type returned by the function

            Arguments
            - <V1>: the type returned by the first future
            - <V2>: the type returned by the second future
            - <V3>: the type returned by the third future
            - <V4>: the type returned by the fourth future
            """

            def apply(self, closer: "DeferredCloser", value1: "V1", value2: "V2", value3: "V3", value4: "V4") -> "U":
                """
                Applies this function to four inputs, or throws an exception if unable to do so.
                
                Any objects that are passed to DeferredCloser.eventuallyClose(Object, Executor)
                closer.eventuallyClose() will be closed when the ClosingFuture pipeline is done
                (but not before this method completes), even if this method throws or the pipeline is
                cancelled.
                """
                ...


        class AsyncClosingFunction4:
            """
            A function that returns a ClosingFuture when applied to the values of the four
            futures passed to .whenAllSucceed(ClosingFuture, ClosingFuture, ClosingFuture,
            ClosingFuture).
            
            Type `<U>`: the type returned by the function

            Arguments
            - <V1>: the type returned by the first future
            - <V2>: the type returned by the second future
            - <V3>: the type returned by the third future
            - <V4>: the type returned by the fourth future
            """

            def apply(self, closer: "DeferredCloser", value1: "V1", value2: "V2", value3: "V3", value4: "V4") -> "ClosingFuture"["U"]:
                """
                Applies this function to four inputs, or throws an exception if unable to do so.
                
                Any objects that are passed to DeferredCloser.eventuallyClose(Object, Executor)
                closer.eventuallyClose() will be closed when the ClosingFuture pipeline is done
                (but not before this method completes), even if this method throws or the pipeline is
                cancelled.
                """
                ...


    class Combiner5(Combiner):
        """
        A generic Combiner that lets you use a lambda or method reference to combine five
        ClosingFutures. Use .whenAllSucceed(ClosingFuture, ClosingFuture, ClosingFuture,
        ClosingFuture, ClosingFuture) to start this combination.

        Arguments
        - <V1>: the type returned by the first future
        - <V2>: the type returned by the second future
        - <V3>: the type returned by the third future
        - <V4>: the type returned by the fourth future
        - <V5>: the type returned by the fifth future
        """

        def call(self, function: "ClosingFunction5"["V1", "V2", "V3", "V4", "V5", "U"], executor: "Executor") -> "ClosingFuture"["U"]:
            """
            Returns a new `ClosingFuture` pipeline step derived from the inputs by applying a
            combining function to their values. The function can use a DeferredCloser to capture
            objects to be closed when the pipeline is done.
            
            If this combiner was returned by .whenAllSucceed(ClosingFuture, ClosingFuture,
            ClosingFuture, ClosingFuture, ClosingFuture) and any of the inputs fail, so will the
            returned step.
            
            If the function throws a `CancellationException`, the pipeline will be cancelled.
            
            If the function throws an `ExecutionException`, the cause of the thrown `ExecutionException` will be extracted and used as the failure of the derived step.
            """
            ...


        def callAsync(self, function: "AsyncClosingFunction5"["V1", "V2", "V3", "V4", "V5", "U"], executor: "Executor") -> "ClosingFuture"["U"]:
            """
            Returns a new `ClosingFuture` pipeline step derived from the inputs by applying a
            `ClosingFuture`-returning function to their values. The function can use a DeferredCloser to capture objects to be closed when the pipeline is done (other than those
            captured by the returned ClosingFuture).
            
            If this combiner was returned by .whenAllSucceed(ClosingFuture, ClosingFuture,
            ClosingFuture, ClosingFuture, ClosingFuture) and any of the inputs fail, so will the
            returned step.
            
            If the function throws a `CancellationException`, the pipeline will be cancelled.
            
            If the function throws an `ExecutionException`, the cause of the thrown `ExecutionException` will be extracted and used as the failure of the derived step.
            
            If the function throws any other exception, it will be used as the failure of the derived
            step.
            
            If an exception is thrown after the function creates a `ClosingFuture`, then none of
            the closeable objects in that `ClosingFuture` will be closed.
            
            Usage guidelines for this method:
            
            
              - Use this method only when calling an API that returns a ListenableFuture or a
                  `ClosingFuture`. If possible, prefer calling .call(CombiningCallable,
                  Executor) instead, with a function that returns the next value directly.
              - Call DeferredCloser.eventuallyClose(Object, Executor) closer.eventuallyClose()
                  for every closeable object this step creates in order to capture it for later closing.
              - Return a `ClosingFuture`. To turn a ListenableFuture into a `ClosingFuture` call .from(ListenableFuture).
            
            
            The same warnings about doing heavyweight operations within ClosingFuture.transformAsync(AsyncClosingFunction, Executor) apply here.
            """
            ...


        class ClosingFunction5:
            """
            A function that returns a value when applied to the values of the five futures passed to
            .whenAllSucceed(ClosingFuture, ClosingFuture, ClosingFuture, ClosingFuture,
            ClosingFuture).
            
            Type `<U>`: the type returned by the function

            Arguments
            - <V1>: the type returned by the first future
            - <V2>: the type returned by the second future
            - <V3>: the type returned by the third future
            - <V4>: the type returned by the fourth future
            - <V5>: the type returned by the fifth future
            """

            def apply(self, closer: "DeferredCloser", value1: "V1", value2: "V2", value3: "V3", value4: "V4", value5: "V5") -> "U":
                """
                Applies this function to five inputs, or throws an exception if unable to do so.
                
                Any objects that are passed to DeferredCloser.eventuallyClose(Object, Executor)
                closer.eventuallyClose() will be closed when the ClosingFuture pipeline is done
                (but not before this method completes), even if this method throws or the pipeline is
                cancelled.
                """
                ...


        class AsyncClosingFunction5:
            """
            A function that returns a ClosingFuture when applied to the values of the five
            futures passed to .whenAllSucceed(ClosingFuture, ClosingFuture, ClosingFuture,
            ClosingFuture, ClosingFuture).
            
            Type `<U>`: the type returned by the function

            Arguments
            - <V1>: the type returned by the first future
            - <V2>: the type returned by the second future
            - <V3>: the type returned by the third future
            - <V4>: the type returned by the fourth future
            - <V5>: the type returned by the fifth future
            """

            def apply(self, closer: "DeferredCloser", value1: "V1", value2: "V2", value3: "V3", value4: "V4", value5: "V5") -> "ClosingFuture"["U"]:
                """
                Applies this function to five inputs, or throws an exception if unable to do so.
                
                Any objects that are passed to DeferredCloser.eventuallyClose(Object, Executor)
                closer.eventuallyClose() will be closed when the ClosingFuture pipeline is done
                (but not before this method completes), even if this method throws or the pipeline is
                cancelled.
                """
                ...



"""
Python module generated from Java source file com.google.common.util.concurrent.Futures

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.base import Function
from com.google.common.base import MoreObjects
from com.google.common.base import Preconditions
from com.google.common.collect import ImmutableList
from com.google.common.util.concurrent import *
from com.google.common.util.concurrent.CollectionFuture import ListFuture
from com.google.common.util.concurrent.ImmediateFuture import ImmediateCancelledFuture
from com.google.common.util.concurrent.ImmediateFuture import ImmediateFailedFuture
from com.google.common.util.concurrent.internal import InternalFutureFailureAccess
from com.google.common.util.concurrent.internal import InternalFutures
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations.concurrent import LazyInit
from java.time import Duration
from java.util.concurrent import Callable
from java.util.concurrent import CancellationException
from java.util.concurrent import ExecutionException
from java.util.concurrent import Executor
from java.util.concurrent import Future
from java.util.concurrent import RejectedExecutionException
from java.util.concurrent import ScheduledExecutorService
from java.util.concurrent import TimeUnit
from java.util.concurrent import TimeoutException
from java.util.concurrent.atomic import AtomicInteger
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Futures(GwtFuturesCatchingSpecialization):
    """
    Static utility methods pertaining to the Future interface.
    
    Many of these methods use the ListenableFuture API; consult the Guava User Guide
    article on <a href="https://github.com/google/guava/wiki/ListenableFutureExplained">`ListenableFuture`</a>.
    
    The main purpose of `ListenableFuture` is to help you chain together a graph of
    asynchronous operations. You can chain them together manually with calls to methods like Futures.transform(ListenableFuture, Function, Executor) Futures.transform, but you will often
    find it easier to use a framework. Frameworks automate the process, often adding features like
    monitoring, debugging, and cancellation. Examples of frameworks include:
    
    
      - <a href="https://dagger.dev/producers.html">Dagger Producers</a>
    
    
    If you do chain your operations manually, you may want to use FluentFuture.

    Author(s)
    - Sven Mawson

    Since
    - 1.0
    """

    @staticmethod
    def immediateFuture(value: "V") -> "ListenableFuture"["V"]:
        """
        Creates a `ListenableFuture` which has its value set immediately upon construction. The
        getters just return the value. This `Future` can't be canceled or timed out and its
        `isDone()` method always returns `True`.
        """
        ...


    @staticmethod
    def immediateVoidFuture() -> "ListenableFuture"["Void"]:
        """
        Returns a successful `ListenableFuture<Void>`. This method is equivalent to `immediateFuture(null)` except that it is restricted to produce futures of type `Void`.

        Since
        - 29.0
        """
        ...


    @staticmethod
    def immediateFailedFuture(throwable: "Throwable") -> "ListenableFuture"["V"]:
        """
        Returns a `ListenableFuture` which has an exception set immediately upon construction.
        
        The returned `Future` can't be cancelled, and its `isDone()` method always
        returns `True`. Calling `get()` will immediately throw the provided `Throwable` wrapped in an `ExecutionException`.
        """
        ...


    @staticmethod
    def immediateCancelledFuture() -> "ListenableFuture"["V"]:
        """
        Creates a `ListenableFuture` which is cancelled immediately upon construction, so that
        `isCancelled()` always returns `True`.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def submit(callable: "Callable"["O"], executor: "Executor") -> "ListenableFuture"["O"]:
        """
        Executes `callable` on the specified `executor`, returning a `Future`.

        Raises
        - RejectedExecutionException: if the task cannot be scheduled for execution

        Since
        - 28.2
        """
        ...


    @staticmethod
    def submit(runnable: "Runnable", executor: "Executor") -> "ListenableFuture"["Void"]:
        """
        Executes `runnable` on the specified `executor`, returning a `Future` that
        will complete after execution.

        Raises
        - RejectedExecutionException: if the task cannot be scheduled for execution

        Since
        - 28.2
        """
        ...


    @staticmethod
    def submitAsync(callable: "AsyncCallable"["O"], executor: "Executor") -> "ListenableFuture"["O"]:
        """
        Executes `callable` on the specified `executor`, returning a `Future`.

        Raises
        - RejectedExecutionException: if the task cannot be scheduled for execution

        Since
        - 23.0
        """
        ...


    @staticmethod
    def scheduleAsync(callable: "AsyncCallable"["O"], delay: "Duration", executorService: "ScheduledExecutorService") -> "ListenableFuture"["O"]:
        """
        Schedules `callable` on the specified `executor`, returning a `Future`.

        Raises
        - RejectedExecutionException: if the task cannot be scheduled for execution

        Since
        - 28.0
        """
        ...


    @staticmethod
    def scheduleAsync(callable: "AsyncCallable"["O"], delay: int, timeUnit: "TimeUnit", executorService: "ScheduledExecutorService") -> "ListenableFuture"["O"]:
        """
        Schedules `callable` on the specified `executor`, returning a `Future`.

        Raises
        - RejectedExecutionException: if the task cannot be scheduled for execution

        Since
        - 23.0
        """
        ...


    @staticmethod
    def catching(input: "ListenableFuture"["V"], exceptionType: type["X"], fallback: "Function"["X", "V"], executor: "Executor") -> "ListenableFuture"["V"]:
        """
        Returns a `Future` whose result is taken from the given primary `input` or, if the
        primary input fails with the given `exceptionType`, from the result provided by the
        `fallback`. Function.apply is not invoked until the primary input has failed, so
        if the primary input succeeds, it is never invoked. If, during the invocation of `fallback`, an exception is thrown, this exception is used as the result of the output `Future`.
        
        Usage example:
        
        ````ListenableFuture<Integer> fetchCounterFuture = ...;
        
        // Falling back to a zero counter in case an exception happens when
        // processing the RPC to fetch counters.
        ListenableFuture<Integer> faultTolerantFuture = Futures.catching(
            fetchCounterFuture, FetchException.class, x -> 0, directExecutor());````
        
        When selecting an executor, note that `directExecutor` is dangerous in some cases. See
        the warnings the MoreExecutors.directExecutor documentation.

        Arguments
        - input: the primary input `Future`
        - exceptionType: the exception type that triggers use of `fallback`. The exception
            type is matched against the input's exception. "The input's exception" means the cause of
            the ExecutionException thrown by `input.get()` or, if `get()` throws a
            different kind of exception, that exception itself. To avoid hiding bugs and other
            unrecoverable errors, callers should prefer more specific types, avoiding `Throwable.class` in particular.
        - fallback: the Function to be called if `input` fails with the expected
            exception type. The function's argument is the input's exception. "The input's exception"
            means the cause of the ExecutionException thrown by `input.get()` or, if
            `get()` throws a different kind of exception, that exception itself.
        - executor: the executor that runs `fallback` if `input` fails

        Since
        - 19.0
        """
        ...


    @staticmethod
    def catchingAsync(input: "ListenableFuture"["V"], exceptionType: type["X"], fallback: "AsyncFunction"["X", "V"], executor: "Executor") -> "ListenableFuture"["V"]:
        """
        Returns a `Future` whose result is taken from the given primary `input` or, if the
        primary input fails with the given `exceptionType`, from the result provided by the
        `fallback`. AsyncFunction.apply is not invoked until the primary input has
        failed, so if the primary input succeeds, it is never invoked. If, during the invocation of
        `fallback`, an exception is thrown, this exception is used as the result of the output
        `Future`.
        
        Usage examples:
        
        ````ListenableFuture<Integer> fetchCounterFuture = ...;
        
        // Falling back to a zero counter in case an exception happens when
        // processing the RPC to fetch counters.
        ListenableFuture<Integer> faultTolerantFuture = Futures.catchingAsync(
            fetchCounterFuture, FetchException.class, x -> immediateFuture(0), directExecutor());````
        
        The fallback can also choose to propagate the original exception when desired:
        
        ````ListenableFuture<Integer> fetchCounterFuture = ...;
        
        // Falling back to a zero counter only in case the exception was a
        // TimeoutException.
        ListenableFuture<Integer> faultTolerantFuture = Futures.catchingAsync(
            fetchCounterFuture,
            FetchException.class,
            e -> {
              if (omitDataOnFetchFailure) {
                return immediateFuture(0);`
              throw e;
            },
            directExecutor());
        }```
        
        When selecting an executor, note that `directExecutor` is dangerous in some cases. See
        the warnings the MoreExecutors.directExecutor documentation.

        Arguments
        - input: the primary input `Future`
        - exceptionType: the exception type that triggers use of `fallback`. The exception
            type is matched against the input's exception. "The input's exception" means the cause of
            the ExecutionException thrown by `input.get()` or, if `get()` throws a
            different kind of exception, that exception itself. To avoid hiding bugs and other
            unrecoverable errors, callers should prefer more specific types, avoiding `Throwable.class` in particular.
        - fallback: the AsyncFunction to be called if `input` fails with the expected
            exception type. The function's argument is the input's exception. "The input's exception"
            means the cause of the ExecutionException thrown by `input.get()` or, if
            `get()` throws a different kind of exception, that exception itself.
        - executor: the executor that runs `fallback` if `input` fails

        Since
        - 19.0 (similar functionality in 14.0 as `withFallback`)
        """
        ...


    @staticmethod
    def withTimeout(delegate: "ListenableFuture"["V"], time: "Duration", scheduledExecutor: "ScheduledExecutorService") -> "ListenableFuture"["V"]:
        """
        Returns a future that delegates to another but will finish early (via a TimeoutException wrapped in an ExecutionException) if the specified duration expires.
        
        The delegate future is interrupted and cancelled if it times out.

        Arguments
        - delegate: The future to delegate to.
        - time: when to time out the future
        - scheduledExecutor: The executor service to enforce the timeout.

        Since
        - 28.0
        """
        ...


    @staticmethod
    def withTimeout(delegate: "ListenableFuture"["V"], time: int, unit: "TimeUnit", scheduledExecutor: "ScheduledExecutorService") -> "ListenableFuture"["V"]:
        """
        Returns a future that delegates to another but will finish early (via a TimeoutException wrapped in an ExecutionException) if the specified duration expires.
        
        The delegate future is interrupted and cancelled if it times out.

        Arguments
        - delegate: The future to delegate to.
        - time: when to time out the future
        - unit: the time unit of the time parameter
        - scheduledExecutor: The executor service to enforce the timeout.

        Since
        - 19.0
        """
        ...


    @staticmethod
    def transformAsync(input: "ListenableFuture"["I"], function: "AsyncFunction"["I", "O"], executor: "Executor") -> "ListenableFuture"["O"]:
        """
        Returns a new `Future` whose result is asynchronously derived from the result of the
        given `Future`. If the given `Future` fails, the returned `Future` fails with
        the same exception (and the function is not invoked).
        
        More precisely, the returned `Future` takes its result from a `Future` produced
        by applying the given `AsyncFunction` to the result of the original `Future`.
        Example usage:
        
        ````ListenableFuture<RowKey> rowKeyFuture = indexService.lookUp(query);
        ListenableFuture<QueryResult> queryFuture =
            transformAsync(rowKeyFuture, dataService::readFuture, executor);````
        
        When selecting an executor, note that `directExecutor` is dangerous in some cases. See
        the warnings the MoreExecutors.directExecutor documentation.
        
        The returned `Future` attempts to keep its cancellation state in sync with that of the
        input future and that of the future returned by the chain function. That is, if the returned
        `Future` is cancelled, it will attempt to cancel the other two, and if either of the
        other two is cancelled, the returned `Future` will receive a callback in which it will
        attempt to cancel itself.

        Arguments
        - input: The future to transform
        - function: A function to transform the result of the input future to the result of the
            output future
        - executor: Executor to run the function in.

        Returns
        - A future that holds result of the function (if the input succeeded) or the original
            input's failure (if not)

        Since
        - 19.0 (in 11.0 as `transform`)
        """
        ...


    @staticmethod
    def transform(input: "ListenableFuture"["I"], function: "Function"["I", "O"], executor: "Executor") -> "ListenableFuture"["O"]:
        """
        Returns a new `Future` whose result is derived from the result of the given `Future`. If `input` fails, the returned `Future` fails with the same exception (and
        the function is not invoked). Example usage:
        
        ````ListenableFuture<QueryResult> queryFuture = ...;
        ListenableFuture<List<Row>> rowsFuture =
            transform(queryFuture, QueryResult::getRows, executor);````
        
        When selecting an executor, note that `directExecutor` is dangerous in some cases. See
        the warnings the MoreExecutors.directExecutor documentation.
        
        The returned `Future` attempts to keep its cancellation state in sync with that of the
        input future. That is, if the returned `Future` is cancelled, it will attempt to cancel
        the input, and if the input is cancelled, the returned `Future` will receive a callback
        in which it will attempt to cancel itself.
        
        An example use of this method is to convert a serializable object returned from an RPC into
        a POJO.

        Arguments
        - input: The future to transform
        - function: A Function to transform the results of the provided future to the results of
            the returned future.
        - executor: Executor to run the function in.

        Returns
        - A future that holds result of the transformation.

        Since
        - 9.0 (in 2.0 as `compose`)
        """
        ...


    @staticmethod
    def lazyTransform(input: "Future"["I"], function: "Function"["I", "O"]) -> "Future"["O"]:
        """
        Like .transform(ListenableFuture, Function, Executor) except that the transformation
        `function` is invoked on each call to Future.get() get() on the returned future.
        
        The returned `Future` reflects the input's cancellation state directly, and any
        attempt to cancel the returned Future is likewise passed through to the input Future.
        
        Note that calls to Future.get(long, TimeUnit) timed get only apply the timeout
        to the execution of the underlying `Future`, *not* to the execution of the
        transformation function.
        
        The primary audience of this method is callers of `transform` who don't have a `ListenableFuture` available and do not mind repeated, lazy function evaluation.

        Arguments
        - input: The future to transform
        - function: A Function to transform the results of the provided future to the results of
            the returned future.

        Returns
        - A future that returns the result of the transformation.

        Since
        - 10.0
        """
        ...


    @staticmethod
    def allAsList(*futures: Tuple["ListenableFuture"["V"], ...]) -> "ListenableFuture"[list["V"]]:
        """
        Creates a new `ListenableFuture` whose value is a list containing the values of all its
        input futures, if all succeed.
        
        The list of results is in the same order as the input list.
        
        This differs from .successfulAsList(ListenableFuture[]) in that it will return a
        failed future if any of the items fails.
        
        Canceling this future will attempt to cancel all the component futures, and if any of the
        provided futures fails or is canceled, this one is, too.

        Arguments
        - futures: futures to combine

        Returns
        - a future that provides a list of the results of the component futures

        Since
        - 10.0
        """
        ...


    @staticmethod
    def allAsList(futures: Iterable["ListenableFuture"["V"]]) -> "ListenableFuture"[list["V"]]:
        """
        Creates a new `ListenableFuture` whose value is a list containing the values of all its
        input futures, if all succeed.
        
        The list of results is in the same order as the input list.
        
        This differs from .successfulAsList(Iterable) in that it will return a failed future
        if any of the items fails.
        
        Canceling this future will attempt to cancel all the component futures, and if any of the
        provided futures fails or is canceled, this one is, too.

        Arguments
        - futures: futures to combine

        Returns
        - a future that provides a list of the results of the component futures

        Since
        - 10.0
        """
        ...


    @staticmethod
    def whenAllComplete(*futures: Tuple["ListenableFuture"["V"], ...]) -> "FutureCombiner"["V"]:
        """
        Creates a FutureCombiner that processes the completed futures whether or not they're
        successful.
        
        Any failures from the input futures will not be propagated to the returned future.

        Since
        - 20.0
        """
        ...


    @staticmethod
    def whenAllComplete(futures: Iterable["ListenableFuture"["V"]]) -> "FutureCombiner"["V"]:
        """
        Creates a FutureCombiner that processes the completed futures whether or not they're
        successful.
        
        Any failures from the input futures will not be propagated to the returned future.

        Since
        - 20.0
        """
        ...


    @staticmethod
    def whenAllSucceed(*futures: Tuple["ListenableFuture"["V"], ...]) -> "FutureCombiner"["V"]:
        """
        Creates a FutureCombiner requiring that all passed in futures are successful.
        
        If any input fails, the returned future fails immediately.

        Since
        - 20.0
        """
        ...


    @staticmethod
    def whenAllSucceed(futures: Iterable["ListenableFuture"["V"]]) -> "FutureCombiner"["V"]:
        """
        Creates a FutureCombiner requiring that all passed in futures are successful.
        
        If any input fails, the returned future fails immediately.

        Since
        - 20.0
        """
        ...


    @staticmethod
    def nonCancellationPropagating(future: "ListenableFuture"["V"]) -> "ListenableFuture"["V"]:
        """
        Returns a `ListenableFuture` whose result is set from the supplied future when it
        completes. Cancelling the supplied future will also cancel the returned future, but cancelling
        the returned future will have no effect on the supplied future.

        Since
        - 15.0
        """
        ...


    @staticmethod
    def successfulAsList(*futures: Tuple["ListenableFuture"["V"], ...]) -> "ListenableFuture"[list["V"]]:
        """
        Creates a new `ListenableFuture` whose value is a list containing the values of all its
        successful input futures. The list of results is in the same order as the input list, and if
        any of the provided futures fails or is canceled, its corresponding position will contain
        `null` (which is indistinguishable from the future having a successful value of `null`).
        
        The list of results is in the same order as the input list.
        
        This differs from .allAsList(ListenableFuture[]) in that it's tolerant of failed
        futures for any of the items, representing them as `null` in the result list.
        
        Canceling this future will attempt to cancel all the component futures.

        Arguments
        - futures: futures to combine

        Returns
        - a future that provides a list of the results of the component futures

        Since
        - 10.0
        """
        ...


    @staticmethod
    def successfulAsList(futures: Iterable["ListenableFuture"["V"]]) -> "ListenableFuture"[list["V"]]:
        """
        Creates a new `ListenableFuture` whose value is a list containing the values of all its
        successful input futures. The list of results is in the same order as the input list, and if
        any of the provided futures fails or is canceled, its corresponding position will contain
        `null` (which is indistinguishable from the future having a successful value of `null`).
        
        The list of results is in the same order as the input list.
        
        This differs from .allAsList(Iterable) in that it's tolerant of failed futures for
        any of the items, representing them as `null` in the result list.
        
        Canceling this future will attempt to cancel all the component futures.

        Arguments
        - futures: futures to combine

        Returns
        - a future that provides a list of the results of the component futures

        Since
        - 10.0
        """
        ...


    @staticmethod
    def inCompletionOrder(futures: Iterable["ListenableFuture"["T"]]) -> "ImmutableList"["ListenableFuture"["T"]]:
        """
        Returns a list of delegate futures that correspond to the futures received in the order that
        they complete. Delegate futures return the same value or throw the same exception as the
        corresponding input future returns/throws.
        
        "In the order that they complete" means, for practical purposes, about what you would
        expect, but there are some subtleties. First, we do guarantee that, if the output future at
        index n is done, the output future at index n-1 is also done. (But as usual with futures, some
        listeners for future n may complete before some for future n-1.) However, it is possible, if
        one input completes with result X and another later with result Y, for Y to come before X in
        the output future list. (Such races are impossible to solve without global synchronization of
        all future completions. And they should have little practical impact.)
        
        Cancelling a delegate future propagates to input futures once all the delegates complete,
        either from cancellation or because an input future has completed. If N futures are passed in,
        and M delegates are cancelled, the remaining M input futures will be cancelled once N - M of
        the input futures complete. If all the delegates are cancelled, all the input futures will be
        too.

        Since
        - 17.0
        """
        ...


    @staticmethod
    def addCallback(future: "ListenableFuture"["V"], callback: "FutureCallback"["V"], executor: "Executor") -> None:
        """
        Registers separate success and failure callbacks to be run when the `Future`'s
        computation is java.util.concurrent.Future.isDone() complete or, if the
        computation is already complete, immediately.
        
        The callback is run on `executor`. There is no guaranteed ordering of execution of
        callbacks, but any callback added through this method is guaranteed to be called once the
        computation is complete.
        
        Exceptions thrown by a `callback` will be propagated up to the executor. Any exception
        thrown during `Executor.execute` (e.g., a `RejectedExecutionException` or an
        exception thrown by MoreExecutors.directExecutor direct execution) will be caught
        and logged.
        
        Example:
        
        ````ListenableFuture<QueryResult> future = ...;
        Executor e = ...
        addCallback(future,
            new FutureCallback<QueryResult>() {
              public void onSuccess(QueryResult result) {
                storeInCache(result);`
              public void onFailure(Throwable t) {
                reportError(t);
              }
            }, e);
        }```
        
        When selecting an executor, note that `directExecutor` is dangerous in some cases. See
        the warnings the MoreExecutors.directExecutor documentation.
        
        For a more general interface to attach a completion listener to a `Future`, see ListenableFuture.addListener addListener.

        Arguments
        - future: The future attach the callback to.
        - callback: The callback to invoke when `future` is completed.
        - executor: The executor to run `callback` when the future completes.

        Since
        - 10.0
        """
        ...


    @staticmethod
    def getDone(future: "Future"["V"]) -> "V":
        """
        Returns the result of the input `Future`, which must have already completed.
        
        The benefits of this method are twofold. First, the name "getDone" suggests to readers that
        the `Future` is already done. Second, if buggy code calls `getDone` on a `Future` that is still pending, the program will throw instead of block. This can be important
        for APIs like .whenAllComplete whenAllComplete(...)`.`FutureCombiner.call(Callable, Executor) call(...), where it is easy to use a new input from
        the `call` implementation but forget to add it to the arguments of `whenAllComplete`.
        
        If you are looking for a method to determine whether a given `Future` is done, use the
        instance method Future.isDone().

        Raises
        - ExecutionException: if the `Future` failed with an exception
        - CancellationException: if the `Future` was cancelled
        - IllegalStateException: if the `Future` is not done

        Since
        - 20.0
        """
        ...


    @staticmethod
    def getChecked(future: "Future"["V"], exceptionClass: type["X"]) -> "V":
        """
        Returns the result of Future.get(), converting most exceptions to a new instance of the
        given checked exception type. This reduces boilerplate for a common use of `Future` in
        which it is unnecessary to programmatically distinguish between exception types or to extract
        other information from the exception instance.
        
        Exceptions from `Future.get` are treated as follows:
        
        
          - Any ExecutionException has its *cause* wrapped in an `X` if the cause
              is a checked exception, an UncheckedExecutionException if the cause is a `RuntimeException`, or an ExecutionError if the cause is an `Error`.
          - Any InterruptedException is wrapped in an `X` (after restoring the
              interrupt).
          - Any CancellationException is propagated untouched, as is any other RuntimeException (though `get` implementations are discouraged from throwing such
              exceptions).
        
        
        The overall principle is to continue to treat every checked exception as a checked
        exception, every unchecked exception as an unchecked exception, and every error as an error. In
        addition, the cause of any `ExecutionException` is wrapped in order to ensure that the
        new stack trace matches that of the current thread.
        
        Instances of `exceptionClass` are created by choosing an arbitrary public constructor
        that accepts zero or more arguments, all of type `String` or `Throwable`
        (preferring constructors with at least one `String`, then preferring constructors with at
        least one `Throwable`) and calling the constructor via reflection. If the exception did
        not already have a cause, one is set by calling Throwable.initCause(Throwable) on it.
        If no such constructor exists, an `IllegalArgumentException` is thrown.

        Raises
        - X: if `get` throws any checked exception except for an `ExecutionException`
            whose cause is not itself a checked exception
        - UncheckedExecutionException: if `get` throws an `ExecutionException` with a
            `RuntimeException` as its cause
        - ExecutionError: if `get` throws an `ExecutionException` with an `Error` as its cause
        - CancellationException: if `get` throws a `CancellationException`
        - IllegalArgumentException: if `exceptionClass` extends `RuntimeException` or
            does not have a suitable constructor

        Since
        - 19.0 (in 10.0 as `get`)
        """
        ...


    @staticmethod
    def getChecked(future: "Future"["V"], exceptionClass: type["X"], timeout: "Duration") -> "V":
        """
        Returns the result of Future.get(long, TimeUnit), converting most exceptions to a new
        instance of the given checked exception type. This reduces boilerplate for a common use of
        `Future` in which it is unnecessary to programmatically distinguish between exception
        types or to extract other information from the exception instance.
        
        Exceptions from `Future.get` are treated as follows:
        
        
          - Any ExecutionException has its *cause* wrapped in an `X` if the cause
              is a checked exception, an UncheckedExecutionException if the cause is a `RuntimeException`, or an ExecutionError if the cause is an `Error`.
          - Any InterruptedException is wrapped in an `X` (after restoring the
              interrupt).
          - Any TimeoutException is wrapped in an `X`.
          - Any CancellationException is propagated untouched, as is any other RuntimeException (though `get` implementations are discouraged from throwing such
              exceptions).
        
        
        The overall principle is to continue to treat every checked exception as a checked
        exception, every unchecked exception as an unchecked exception, and every error as an error. In
        addition, the cause of any `ExecutionException` is wrapped in order to ensure that the
        new stack trace matches that of the current thread.
        
        Instances of `exceptionClass` are created by choosing an arbitrary public constructor
        that accepts zero or more arguments, all of type `String` or `Throwable`
        (preferring constructors with at least one `String`, then preferring constructors with at
        least one `Throwable`) and calling the constructor via reflection. If the exception did
        not already have a cause, one is set by calling Throwable.initCause(Throwable) on it.
        If no such constructor exists, an `IllegalArgumentException` is thrown.

        Raises
        - X: if `get` throws any checked exception except for an `ExecutionException`
            whose cause is not itself a checked exception
        - UncheckedExecutionException: if `get` throws an `ExecutionException` with a
            `RuntimeException` as its cause
        - ExecutionError: if `get` throws an `ExecutionException` with an `Error` as its cause
        - CancellationException: if `get` throws a `CancellationException`
        - IllegalArgumentException: if `exceptionClass` extends `RuntimeException` or
            does not have a suitable constructor

        Since
        - 28.0
        """
        ...


    @staticmethod
    def getChecked(future: "Future"["V"], exceptionClass: type["X"], timeout: int, unit: "TimeUnit") -> "V":
        """
        Returns the result of Future.get(long, TimeUnit), converting most exceptions to a new
        instance of the given checked exception type. This reduces boilerplate for a common use of
        `Future` in which it is unnecessary to programmatically distinguish between exception
        types or to extract other information from the exception instance.
        
        Exceptions from `Future.get` are treated as follows:
        
        
          - Any ExecutionException has its *cause* wrapped in an `X` if the cause
              is a checked exception, an UncheckedExecutionException if the cause is a `RuntimeException`, or an ExecutionError if the cause is an `Error`.
          - Any InterruptedException is wrapped in an `X` (after restoring the
              interrupt).
          - Any TimeoutException is wrapped in an `X`.
          - Any CancellationException is propagated untouched, as is any other RuntimeException (though `get` implementations are discouraged from throwing such
              exceptions).
        
        
        The overall principle is to continue to treat every checked exception as a checked
        exception, every unchecked exception as an unchecked exception, and every error as an error. In
        addition, the cause of any `ExecutionException` is wrapped in order to ensure that the
        new stack trace matches that of the current thread.
        
        Instances of `exceptionClass` are created by choosing an arbitrary public constructor
        that accepts zero or more arguments, all of type `String` or `Throwable`
        (preferring constructors with at least one `String`) and calling the constructor via
        reflection. If the exception did not already have a cause, one is set by calling Throwable.initCause(Throwable) on it. If no such constructor exists, an `IllegalArgumentException` is thrown.

        Raises
        - X: if `get` throws any checked exception except for an `ExecutionException`
            whose cause is not itself a checked exception
        - UncheckedExecutionException: if `get` throws an `ExecutionException` with a
            `RuntimeException` as its cause
        - ExecutionError: if `get` throws an `ExecutionException` with an `Error` as its cause
        - CancellationException: if `get` throws a `CancellationException`
        - IllegalArgumentException: if `exceptionClass` extends `RuntimeException` or
            does not have a suitable constructor

        Since
        - 19.0 (in 10.0 as `get` and with different parameter order)
        """
        ...


    @staticmethod
    def getUnchecked(future: "Future"["V"]) -> "V":
        """
        Returns the result of calling Future.get() uninterruptibly on a task known not to throw
        a checked exception. This makes `Future` more suitable for lightweight, fast-running
        tasks that, barring bugs in the code, will not fail. This gives it exception-handling behavior
        similar to that of `ForkJoinTask.join`.
        
        Exceptions from `Future.get` are treated as follows:
        
        
          - Any ExecutionException has its *cause* wrapped in an UncheckedExecutionException (if the cause is an `Exception`) or ExecutionError (if the cause is an `Error`).
          - Any InterruptedException causes a retry of the `get` call. The interrupt is
              restored before `getUnchecked` returns.
          - Any CancellationException is propagated untouched. So is any other RuntimeException (`get` implementations are discouraged from throwing such
              exceptions).
        
        
        The overall principle is to eliminate all checked exceptions: to loop to avoid `InterruptedException`, to pass through `CancellationException`, and to wrap any exception
        from the underlying computation in an `UncheckedExecutionException` or `ExecutionError`.
        
        For an uninterruptible `get` that preserves other exceptions, see Uninterruptibles.getUninterruptibly(Future).

        Raises
        - UncheckedExecutionException: if `get` throws an `ExecutionException` with an
            `Exception` as its cause
        - ExecutionError: if `get` throws an `ExecutionException` with an `Error` as its cause
        - CancellationException: if `get` throws a `CancellationException`

        Since
        - 10.0
        """
        ...


    class FutureCombiner:
        """
        A helper to create a new `ListenableFuture` whose result is generated from a combination
        of input futures.
        
        See .whenAllComplete and .whenAllSucceed for how to instantiate this class.
        
        Example:
        
        ````final ListenableFuture<Instant> loginDateFuture =
            loginService.findLastLoginDate(username);
        final ListenableFuture<List<String>> recentCommandsFuture =
            recentCommandsService.findRecentCommands(username);
        ListenableFuture<UsageHistory> usageFuture =
            Futures.whenAllSucceed(loginDateFuture, recentCommandsFuture)
                .call(
                    () ->
                        new UsageHistory(
                            username,
                            Futures.getDone(loginDateFuture),
                            Futures.getDone(recentCommandsFuture)),
                    executor);````

        Since
        - 20.0
        """

        def callAsync(self, combiner: "AsyncCallable"["C"], executor: "Executor") -> "ListenableFuture"["C"]:
            """
            Creates the ListenableFuture which will return the result of calling AsyncCallable.call in `combiner` when all futures complete, using the specified `executor`.
            
            If the combiner throws a `CancellationException`, the returned future will be
            cancelled.
            
            If the combiner throws an `ExecutionException`, the cause of the thrown `ExecutionException` will be extracted and returned as the cause of the new `ExecutionException` that gets thrown by the returned combined future.
            
            Canceling this future will attempt to cancel all the component futures.

            Returns
            - a future whose result is based on `combiner` (or based on the input futures
                passed to `whenAllSucceed`, if that is the method you used to create this `FutureCombiner`). Even if you don't care about the value of the future, you should
                typically check whether it failed: See <a
                href="https://errorprone.info/bugpattern/FutureReturnValueIgnored">https://errorprone.info/bugpattern/FutureReturnValueIgnored</a>.
            """
            ...


        def call(self, combiner: "Callable"["C"], executor: "Executor") -> "ListenableFuture"["C"]:
            """
            Creates the ListenableFuture which will return the result of calling Callable.call in `combiner` when all futures complete, using the specified `executor`.
            
            If the combiner throws a `CancellationException`, the returned future will be
            cancelled.
            
            If the combiner throws an `ExecutionException`, the cause of the thrown `ExecutionException` will be extracted and returned as the cause of the new `ExecutionException` that gets thrown by the returned combined future.
            
            Canceling this future will attempt to cancel all the component futures.

            Returns
            - a future whose result is based on `combiner` (or based on the input futures
                passed to `whenAllSucceed`, if that is the method you used to create this `FutureCombiner`). Even if you don't care about the value of the future, you should
                typically check whether it failed: See <a
                href="https://errorprone.info/bugpattern/FutureReturnValueIgnored">https://errorprone.info/bugpattern/FutureReturnValueIgnored</a>.
            """
            ...


        def run(self, combiner: "Runnable", executor: "Executor") -> "ListenableFuture"[Any]:
            """
            Creates the ListenableFuture which will return the result of running `combiner`
            when all Futures complete. `combiner` will run using `executor`.
            
            If the combiner throws a `CancellationException`, the returned future will be
            cancelled.
            
            Canceling this Future will attempt to cancel all the component futures.

            Returns
            - a future whose result is based on `combiner` (or based on the input futures
                passed to `whenAllSucceed`, if that is the method you used to create this `FutureCombiner`). Even though the future never produces a value other than `null`,
                you should typically check whether it failed: See <a
                href="https://errorprone.info/bugpattern/FutureReturnValueIgnored">https://errorprone.info/bugpattern/FutureReturnValueIgnored</a>.

            Since
            - 23.6
            """
            ...



"""
Python module generated from Java source file com.google.gson.JsonArray

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.gson import *
from com.google.gson.internal import NonNullElementWrapperList
from java.math import BigDecimal
from java.math import BigInteger
from java.util import Iterator
from typing import Any, Callable, Iterable, Tuple


class JsonArray(JsonElement, Iterable):
    """
    A class representing an array type in JSON. An array is a list of JsonElements each of
    which can be of a different type. This is an ordered list, meaning that the order in which
    elements are added is preserved. This class does not support `null` values. If `null`
    is provided as element argument to any of the methods, it is converted to a JsonNull.
    
    `JsonArray` only implements the Iterable interface but not the List
    interface. A `List` view of it can be obtained with .asList().
    
    See the JsonElement documentation for details on how to convert `JsonArray` and
    generally any `JsonElement` from and to JSON.

    Author(s)
    - Joel Leitch
    """

    def __init__(self):
        """
        Creates an empty JsonArray.
        """
        ...


    def __init__(self, capacity: int):
        """
        Creates an empty JsonArray with the desired initial capacity.

        Arguments
        - capacity: initial capacity.

        Raises
        - IllegalArgumentException: if the `capacity` is negative

        Since
        - 2.8.1
        """
        ...


    def deepCopy(self) -> "JsonArray":
        """
        Creates a deep copy of this element and all its children.

        Since
        - 2.8.2
        """
        ...


    def add(self, bool: "Boolean") -> None:
        """
        Adds the specified boolean to self.

        Arguments
        - bool: the boolean that needs to be added to the array.

        Since
        - 2.4
        """
        ...


    def add(self, character: "Character") -> None:
        """
        Adds the specified character to self.

        Arguments
        - character: the character that needs to be added to the array.

        Since
        - 2.4
        """
        ...


    def add(self, number: "Number") -> None:
        """
        Adds the specified number to self.

        Arguments
        - number: the number that needs to be added to the array.

        Since
        - 2.4
        """
        ...


    def add(self, string: str) -> None:
        """
        Adds the specified string to self.

        Arguments
        - string: the string that needs to be added to the array.

        Since
        - 2.4
        """
        ...


    def add(self, element: "JsonElement") -> None:
        """
        Adds the specified element to self.

        Arguments
        - element: the element that needs to be added to the array.
        """
        ...


    def addAll(self, array: "JsonArray") -> None:
        """
        Adds all the elements of the specified array to self.

        Arguments
        - array: the array whose elements need to be added to the array.
        """
        ...


    def set(self, index: int, element: "JsonElement") -> "JsonElement":
        """
        Replaces the element at the specified position in this array with the specified element.

        Arguments
        - index: index of the element to replace
        - element: element to be stored at the specified position

        Returns
        - the element previously at the specified position

        Raises
        - IndexOutOfBoundsException: if the specified index is outside the array bounds
        """
        ...


    def remove(self, element: "JsonElement") -> bool:
        """
        Removes the first occurrence of the specified element from this array, if it is present. If the
        array does not contain the element, it is unchanged.

        Arguments
        - element: element to be removed from this array, if present

        Returns
        - True if this array contained the specified element, False otherwise

        Since
        - 2.3
        """
        ...


    def remove(self, index: int) -> "JsonElement":
        """
        Removes the element at the specified position in this array. Shifts any subsequent elements to
        the left (subtracts one from their indices). Returns the element that was removed from the
        array.

        Arguments
        - index: index the index of the element to be removed

        Returns
        - the element previously at the specified position

        Raises
        - IndexOutOfBoundsException: if the specified index is outside the array bounds

        Since
        - 2.3
        """
        ...


    def contains(self, element: "JsonElement") -> bool:
        """
        Returns True if this array contains the specified element.

        Arguments
        - element: whose presence in this array is to be tested

        Returns
        - True if this array contains the specified element.

        Since
        - 2.3
        """
        ...


    def size(self) -> int:
        """
        Returns the number of elements in the array.

        Returns
        - the number of elements in the array.
        """
        ...


    def isEmpty(self) -> bool:
        """
        Returns True if the array is empty.

        Returns
        - True if the array is empty.

        Since
        - 2.8.7
        """
        ...


    def iterator(self) -> Iterator["JsonElement"]:
        """
        Returns an iterator to navigate the elements of the array. Since the array is an ordered list,
        the iterator navigates the elements in the order they were inserted.

        Returns
        - an iterator to navigate the elements of the array.
        """
        ...


    def get(self, i: int) -> "JsonElement":
        """
        Returns the i-th element of the array.

        Arguments
        - i: the index of the element that is being sought.

        Returns
        - the element present at the i-th index.

        Raises
        - IndexOutOfBoundsException: if `i` is negative or greater than or equal to the
            .size() of the array.
        """
        ...


    def getAsNumber(self) -> "Number":
        """
        Convenience method to get this array as a Number if it contains a single element. This
        method calls JsonElement.getAsNumber() on the element, therefore any of the exceptions
        declared by that method can occur.

        Returns
        - this element as a number if it is single element array.

        Raises
        - IllegalStateException: if the array is empty or has more than one element.
        """
        ...


    def getAsString(self) -> str:
        """
        Convenience method to get this array as a String if it contains a single element. This
        method calls JsonElement.getAsString() on the element, therefore any of the exceptions
        declared by that method can occur.

        Returns
        - this element as a String if it is single element array.

        Raises
        - IllegalStateException: if the array is empty or has more than one element.
        """
        ...


    def getAsDouble(self) -> float:
        """
        Convenience method to get this array as a double if it contains a single element. This method
        calls JsonElement.getAsDouble() on the element, therefore any of the exceptions
        declared by that method can occur.

        Returns
        - this element as a double if it is single element array.

        Raises
        - IllegalStateException: if the array is empty or has more than one element.
        """
        ...


    def getAsBigDecimal(self) -> "BigDecimal":
        """
        Convenience method to get this array as a BigDecimal if it contains a single element.
        This method calls JsonElement.getAsBigDecimal() on the element, therefore any of the
        exceptions declared by that method can occur.

        Returns
        - this element as a BigDecimal if it is single element array.

        Raises
        - IllegalStateException: if the array is empty or has more than one element.

        Since
        - 1.2
        """
        ...


    def getAsBigInteger(self) -> "BigInteger":
        """
        Convenience method to get this array as a BigInteger if it contains a single element.
        This method calls JsonElement.getAsBigInteger() on the element, therefore any of the
        exceptions declared by that method can occur.

        Returns
        - this element as a BigInteger if it is single element array.

        Raises
        - IllegalStateException: if the array is empty or has more than one element.

        Since
        - 1.2
        """
        ...


    def getAsFloat(self) -> float:
        """
        Convenience method to get this array as a float if it contains a single element. This method
        calls JsonElement.getAsFloat() on the element, therefore any of the exceptions declared
        by that method can occur.

        Returns
        - this element as a float if it is single element array.

        Raises
        - IllegalStateException: if the array is empty or has more than one element.
        """
        ...


    def getAsLong(self) -> int:
        """
        Convenience method to get this array as a long if it contains a single element. This method
        calls JsonElement.getAsLong() on the element, therefore any of the exceptions declared
        by that method can occur.

        Returns
        - this element as a long if it is single element array.

        Raises
        - IllegalStateException: if the array is empty or has more than one element.
        """
        ...


    def getAsInt(self) -> int:
        """
        Convenience method to get this array as an integer if it contains a single element. This method
        calls JsonElement.getAsInt() on the element, therefore any of the exceptions declared
        by that method can occur.

        Returns
        - this element as an integer if it is single element array.

        Raises
        - IllegalStateException: if the array is empty or has more than one element.
        """
        ...


    def getAsByte(self) -> int:
        """
        Convenience method to get this array as a primitive byte if it contains a single element. This
        method calls JsonElement.getAsByte() on the element, therefore any of the exceptions
        declared by that method can occur.

        Returns
        - this element as a primitive byte if it is single element array.

        Raises
        - IllegalStateException: if the array is empty or has more than one element.
        """
        ...


    def getAsCharacter(self) -> str:
        """
        Convenience method to get this array as a character if it contains a single element. This
        method calls JsonElement.getAsCharacter() on the element, therefore any of the
        exceptions declared by that method can occur.

        Returns
        - this element as a primitive short if it is single element array.

        Raises
        - IllegalStateException: if the array is empty or has more than one element.

        Deprecated
        - This method is misleading, as it does not get this element as a char but rather as
            a string's first character.
        """
        ...


    def getAsShort(self) -> int:
        """
        Convenience method to get this array as a primitive short if it contains a single element. This
        method calls JsonElement.getAsShort() on the element, therefore any of the exceptions
        declared by that method can occur.

        Returns
        - this element as a primitive short if it is single element array.

        Raises
        - IllegalStateException: if the array is empty or has more than one element.
        """
        ...


    def getAsBoolean(self) -> bool:
        """
        Convenience method to get this array as a boolean if it contains a single element. This method
        calls JsonElement.getAsBoolean() on the element, therefore any of the exceptions
        declared by that method can occur.

        Returns
        - this element as a boolean if it is single element array.

        Raises
        - IllegalStateException: if the array is empty or has more than one element.
        """
        ...


    def asList(self) -> list["JsonElement"]:
        """
        Returns a mutable List view of this `JsonArray`. Changes to the `List` are
        visible in this `JsonArray` and the other way around.
        
        The `List` does not permit `null` elements. Unlike `JsonArray`'s `null` handling, a NullPointerException is thrown when trying to add `null`. Use
        JsonNull for JSON null values.

        Returns
        - mutable `List` view

        Since
        - 2.10
        """
        ...


    def equals(self, o: "Object") -> bool:
        """
        Returns whether the other object is equal to this. This method only considers the other object
        to be equal if it is an instance of `JsonArray` and has equal elements in the same order.
        """
        ...


    def hashCode(self) -> int:
        """
        Returns the hash code of this array. This method calculates the hash code based on the elements
        of this array.
        """
        ...



"""
Python module generated from Java source file com.google.common.cache.RemovalListener

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.cache import *
from typing import Any, Callable, Iterable, Tuple


class RemovalListener:
    """
    An object that can receive a notification when an entry is removed from a cache. The removal
    resulting in notification could have occurred to an entry being manually removed or replaced, or
    due to eviction resulting from timed expiration, exceeding a maximum size, or garbage collection.
    
    An instance may be called concurrently by multiple threads to process different entries.
    Implementations of this interface should avoid performing blocking calls or synchronizing on
    shared resources.
    
    Type `<K>`: the most general type of keys this listener can listen for; for example `Object`
        if any key is acceptable
    
    Type `<V>`: the most general type of values this listener can listen for; for example `Object` if any key is acceptable

    Author(s)
    - Charles Fry

    Since
    - 10.0
    """

    def onRemoval(self, notification: "RemovalNotification"["K", "V"]) -> None:
        ...



"""
Python module generated from Java source file com.google.common.net.HostAndPort

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import CharMatcher
from com.google.common.base import Objects
from com.google.common.base import Strings
from com.google.common.net import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import Immutable
from java.io import Serializable
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class HostAndPort(Serializable):
    """
    An immutable representation of a host and port.
    
    Example usage:
    
    ```
    HostAndPort hp = HostAndPort.fromString("[2001:db8::1]")
        .withDefaultPort(80)
        .requireBracketsForIPv6();
    hp.getHost();   // returns "2001:db8::1"
    hp.getPort();   // returns 80
    hp.toString();  // returns "[2001:db8::1]:80"
    ```
    
    Here are some examples of recognized formats:
    
    
      - example.com
      - example.com:80
      - 192.0.2.1
      - 192.0.2.1:80
      - [2001:db8::1] - .getHost() omits brackets
      - [2001:db8::1]:80 - .getHost() omits brackets
      - 2001:db8::1 - Use .requireBracketsForIPv6() to prohibit this
    
    
    Note that this is not an exhaustive list, because these methods are only concerned with
    brackets, colons, and port numbers. Full validation of the host field (if desired) is the
    caller's responsibility.

    Author(s)
    - Paul Marks

    Since
    - 10.0
    """

    def getHost(self) -> str:
        """
        Returns the portion of this `HostAndPort` instance that should represent the hostname or
        IPv4/IPv6 literal.
        
        A successful parse does not imply any degree of sanity in this field. For additional
        validation, see the HostSpecifier class.

        Since
        - 20.0 (since 10.0 as `getHostText`)
        """
        ...


    def hasPort(self) -> bool:
        """
        Return True if this instance has a defined port.
        """
        ...


    def getPort(self) -> int:
        """
        Get the current port number, failing if no port is defined.

        Returns
        - a validated port number, in the range [0..65535]

        Raises
        - IllegalStateException: if no port is defined. You can use .withDefaultPort(int)
            to prevent this from occurring.
        """
        ...


    def getPortOrDefault(self, defaultPort: int) -> int:
        """
        Returns the current port number, with a default if no port is defined.
        """
        ...


    @staticmethod
    def fromParts(host: str, port: int) -> "HostAndPort":
        """
        Build a HostAndPort instance from separate host and port values.
        
        Note: Non-bracketed IPv6 literals are allowed. Use .requireBracketsForIPv6() to
        prohibit these.

        Arguments
        - host: the host string to parse. Must not contain a port number.
        - port: a port number from [0..65535]

        Returns
        - if parsing was successful, a populated HostAndPort object.

        Raises
        - IllegalArgumentException: if `host` contains a port number, or `port` is out
            of range.
        """
        ...


    @staticmethod
    def fromHost(host: str) -> "HostAndPort":
        """
        Build a HostAndPort instance from a host only.
        
        Note: Non-bracketed IPv6 literals are allowed. Use .requireBracketsForIPv6() to
        prohibit these.

        Arguments
        - host: the host-only string to parse. Must not contain a port number.

        Returns
        - if parsing was successful, a populated HostAndPort object.

        Raises
        - IllegalArgumentException: if `host` contains a port number.

        Since
        - 17.0
        """
        ...


    @staticmethod
    def fromString(hostPortString: str) -> "HostAndPort":
        """
        Split a freeform string into a host and port, without strict validation.
        
        Note that the host-only formats will leave the port field undefined. You can use .withDefaultPort(int) to patch in a default value.

        Arguments
        - hostPortString: the input string to parse.

        Returns
        - if parsing was successful, a populated HostAndPort object.

        Raises
        - IllegalArgumentException: if nothing meaningful could be parsed.
        """
        ...


    def withDefaultPort(self, defaultPort: int) -> "HostAndPort":
        """
        Provide a default port if the parsed string contained only a host.
        
        You can chain this after .fromString(String) to include a port in case the port was
        omitted from the input string. If a port was already provided, then this method is a no-op.

        Arguments
        - defaultPort: a port number, from [0..65535]

        Returns
        - a HostAndPort instance, guaranteed to have a defined port.
        """
        ...


    def requireBracketsForIPv6(self) -> "HostAndPort":
        """
        Generate an error if the host might be a non-bracketed IPv6 literal.
        
        URI formatting requires that IPv6 literals be surrounded by brackets, like "[2001:db8::1]".
        Chain this call after .fromString(String) to increase the strictness of the parser, and
        disallow IPv6 literals that don't contain these brackets.
        
        Note that this parser identifies IPv6 literals solely based on the presence of a colon. To
        perform actual validation of IP addresses, see the InetAddresses.forString(String)
        method.

        Returns
        - `this`, to enable chaining of calls.

        Raises
        - IllegalArgumentException: if bracketless IPv6 is detected.
        """
        ...


    def equals(self, other: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...


    def toString(self) -> str:
        """
        Rebuild the host:port string, including brackets if necessary.
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.CreatureSpawner

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import *
from org.bukkit.spawner import Spawner
from typing import Any, Callable, Iterable, Tuple


class CreatureSpawner(TileState, Spawner):
    """
    Represents a captured state of a creature spawner.
    """

    def setCreatureTypeByName(self, creatureType: str) -> None:
        """
        Set the spawner mob type.

        Arguments
        - creatureType: The creature type's name or null to clear.

        Deprecated
        - magic value, use
        .setSpawnedType(org.bukkit.entity.EntityType).
        """
        ...


    def getCreatureTypeName(self) -> str:
        """
        Get the spawner's creature type.

        Returns
        - The creature type's name if is set.

        Deprecated
        - magic value, use .getSpawnedType().
        """
        ...



"""
Python module generated from Java source file com.google.gson.internal.bind.JsonTreeWriter

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.gson import JsonArray
from com.google.gson import JsonElement
from com.google.gson import JsonNull
from com.google.gson import JsonObject
from com.google.gson import JsonPrimitive
from com.google.gson.internal.bind import *
from com.google.gson.stream import JsonWriter
from java.io import IOException
from java.io import Writer
from java.util import Objects
from typing import Any, Callable, Iterable, Tuple


class JsonTreeWriter(JsonWriter):
    """
    This writer creates a JsonElement.
    """

    def __init__(self):
        ...


    def get(self) -> "JsonElement":
        """
        Returns the top level object produced by this writer.
        """
        ...


    def beginArray(self) -> "JsonWriter":
        ...


    def endArray(self) -> "JsonWriter":
        ...


    def beginObject(self) -> "JsonWriter":
        ...


    def endObject(self) -> "JsonWriter":
        ...


    def name(self, name: str) -> "JsonWriter":
        ...


    def value(self, value: str) -> "JsonWriter":
        ...


    def value(self, value: bool) -> "JsonWriter":
        ...


    def value(self, value: "Boolean") -> "JsonWriter":
        ...


    def value(self, value: float) -> "JsonWriter":
        ...


    def value(self, value: float) -> "JsonWriter":
        ...


    def value(self, value: int) -> "JsonWriter":
        ...


    def value(self, value: "Number") -> "JsonWriter":
        ...


    def nullValue(self) -> "JsonWriter":
        ...


    def jsonValue(self, value: str) -> "JsonWriter":
        ...


    def flush(self) -> None:
        ...


    def close(self) -> None:
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.tokens.Token

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.yaml.snakeyaml.error import Mark
from org.yaml.snakeyaml.error import YAMLException
from org.yaml.snakeyaml.tokens import *
from typing import Any, Callable, Iterable, Tuple


class Token:

    def __init__(self, startMark: "Mark", endMark: "Mark"):
        ...


    def getStartMark(self) -> "Mark":
        ...


    def getEndMark(self) -> "Mark":
        ...


    def getTokenId(self) -> "Token.ID":
        """
        For error reporting.

        Returns
        - ID of this token

        See
        - "class variable 'id' in PyYAML"
        """
        ...


    class ID(Enum):

        Alias = ("<alias>")
        Anchor = ("<anchor>")
        BlockEnd = ("<block end>")
        BlockEntry = ("-")
        BlockMappingStart = ("<block mapping start>")
        BlockSequenceStart = ("<block sequence start>")
        Directive = ("<directive>")
        DocumentEnd = ("<document end>")
        DocumentStart = ("<document start>")
        FlowEntry = (",")
        FlowMappingEnd = ("}")
        FlowMappingStart = ("{")
        FlowSequenceEnd = ("]")
        FlowSequenceStart = ("[")
        Key = ("?")
        Scalar = ("<scalar>")
        StreamEnd = ("<stream end>")
        StreamStart = ("<stream start>")
        Tag = ("<tag>")
        Value = (":")
        Whitespace = ("<whitespace>")
        Comment = ("#")
        Error = ("<error>")


        def toString(self) -> str:
            ...



"""
Python module generated from Java source file java.lang.reflect.Member

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.lang.reflect import *
from typing import Any, Callable, Iterable, Tuple


class Member:
    """
    Member is an interface that reflects identifying information about
    a single member (a field or a method) or a constructor.

    Author(s)
    - Nakul Saraiya

    See
    - Constructor

    Since
    - 1.1
    """

    PUBLIC = 0
    """
    Identifies the set of all public members of a class or interface,
    including inherited members.
    """
    DECLARED = 1
    """
    Identifies the set of declared members of a class or interface.
    Inherited members are not included.
    """


    def getDeclaringClass(self) -> type[Any]:
        """
        Returns the Class object representing the class or interface
        that declares the member or constructor represented by this Member.

        Returns
        - an object representing the declaring class of the
        underlying member
        """
        ...


    def getName(self) -> str:
        """
        Returns the simple name of the underlying member or constructor
        represented by this Member.

        Returns
        - the simple name of the underlying member
        """
        ...


    def getModifiers(self) -> int:
        """
        Returns the Java language modifiers for the member or
        constructor represented by this Member, as an integer.  The
        Modifier class should be used to decode the modifiers in
        the integer.

        Returns
        - the Java language modifiers for the underlying member

        See
        - Modifier
        """
        ...


    def isSynthetic(self) -> bool:
        """
        Returns `True` if this member was introduced by
        the compiler; returns `False` otherwise.

        Returns
        - True if and only if this member was introduced by
        the compiler.

        Since
        - 1.5

        Unknown Tags
        - 13.1 The Form of a Binary
        """
        ...



"""
Python module generated from Java source file org.bukkit.command.PluginIdentifiableCommand

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.command import *
from org.bukkit.plugin import Plugin
from typing import Any, Callable, Iterable, Tuple


class PluginIdentifiableCommand:
    """
    This interface is used by the help system to group commands into
    sub-indexes based on the Plugin they are a part of. Custom command
    implementations will need to implement this interface to have a sub-index
    automatically generated on the plugin's behalf.
    """

    def getPlugin(self) -> "Plugin":
        """
        Gets the owner of this PluginIdentifiableCommand.

        Returns
        - Plugin that owns this PluginIdentifiableCommand.
        """
        ...
